#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.

import mock

from otcextensions.osclient.cbr.v3 import vault
from otcextensions.sdk.cbr.v3 import vault as vaultSDK
from otcextensions.tests.unit.osclient.cbr.v3 import fakes


class TestVault(fakes.TestCBR):

    def setUp(self):
        super(TestVault, self).setUp()

    def test_flatten(self):
        obj = fakes.FakeVault.create_one()

        flat_data = vault._flatten_vault(obj)

        data = (
            flat_data['id'],
            flat_data['name'],
            flat_data['auto_bind'],
            flat_data['auto_expand'],
            flat_data['backup_policy_id'],
            flat_data['created_at'],
            flat_data['description'],
            flat_data['project_id'],
            flat_data['provider_id'],
            flat_data['user_id'],
            flat_data['status'],
            flat_data['operation_type'],
            flat_data['object_type'],
            flat_data['spec_code'],
            flat_data['size'],
            flat_data['consistent_level'],
            flat_data['charging_mode'],
            flat_data['is_auto_pay'],
            flat_data['is_auto_renew'],
            flat_data['bind_rules'],
            flat_data['resources'],
            flat_data['tags']
        )

        cmp_data = (
            obj.id,
            obj.name,
            obj.auto_bind,
            obj.auto_expand,
            obj.backup_policy_id,
            obj.created_at,
            obj.description,
            obj.project_id,
            obj.provider_id,
            obj.user_id,
            obj.billing.status,
            obj.billing.protect_type,
            obj.billing.object_type,
            obj.billing.spec_code,
            obj.billing.size,
            obj.billing.consistent_level,
            obj.billing.charging_mode,
            obj.billing.is_auto_pay,
            obj.billing.is_auto_renew,
            obj.bind_rules["tags"],
            obj.resources,
            obj.tags,
        )

        self.assertEqual(data, cmp_data)

    def test_add_resources_to_vault_output(self):
        obj = fakes.FakeVault.create_one()

        column = ()
        data = ()
        verify_column = (
            'resource_id_1',
            'resource_type_1',
        )
        verify_data = (
            'resource_id',
            'OS::Nova::Server',
        )

        data, column = vault._add_resources_to_vault_obj(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_tags_to_vault_output(self):
        obj = fakes.FakeVault.create_one()

        column = ()
        data = ()
        verify_column = (
            'tags',
        )
        verify_data = (
            ('value=val-tags, key=key-tags',)
        )

        data, column = vault._add_tags_to_vault_obj(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_bind_rules_to_vault_output(self):
        obj = fakes.FakeVault.create_one()

        column = ()
        data = ()
        verify_column = (
            'bind_rules',
        )
        verify_data = (
            ('value=val-bind, key=key-bind',)
        )

        data, column = vault._add_bind_rules_to_vault_obj(obj, data, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_associated_policy_to_vault_output(self):
        obj = {
            'associate_policy': {
                'vault_id': 'cc56d0c6-c0c3-47e6-84cc-d7840dccb706',
                'policy_id': '6359dd6f-4146-42f8-9d7f-fbd6fa740d9f'
            }
        }

        column = ()
        verify_column = (
            'vault_id',
            'policy_id'
        )
        verify_data = (
            'cc56d0c6-c0c3-47e6-84cc-d7840dccb706',
            '6359dd6f-4146-42f8-9d7f-fbd6fa740d9f'
        )

        data, column = vault._add_associated_policy_to_vault_obj(obj, column)

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_add_associated_resources_to_vault_output(self):
        obj = {
            'add_resource_ids': [
                'cc56d0c6-c0c3-47e6-84cc-d7840dccb706',
                '6359dd6f-4146-42f8-9d7f-fbd6fa740d9f'
            ]
        }

        column = ()
        verify_column = (
            'resource_1',
            'resource_2'
        )
        verify_data = (
            'cc56d0c6-c0c3-47e6-84cc-d7840dccb706',
            '6359dd6f-4146-42f8-9d7f-fbd6fa740d9f'
        )

        data, column = vault._add_associated_resources_to_vault_obj(
            obj, column
        )

        self.assertEqual(data, verify_data)
        self.assertEqual(column, verify_column)

    def test_normalize_tags(self):
        tags = [
            'key1=value',
            'key2=',
            'key3'
        ]

        verify_result = [
            {'key': 'key1', 'value': 'value'},
            {'key': 'key2', 'value': ''},
            {'key': 'key3', 'value': ''}
        ]

        result = vault._normalize_tags(tags)

        self.assertEqual(result, verify_result)


class TestListVault(fakes.TestCBR):

    objects = fakes.FakeVault.create_multiple(3)

    columns = ('ID', 'name', 'backup_policy_id', 'description', 'created_at')

    data = []

    for s in objects:
        flat_data = vault._flatten_vault(s)
        data.append((
            flat_data['id'],
            flat_data['name'],
            flat_data['backup_policy_id'],
            flat_data['description'],
            flat_data['created_at'],
        ))

    def setUp(self):
        super(TestListVault, self).setUp()

        self.cmd = vault.ListVaults(self.app, None)

        self.client.vaults = mock.Mock()
        self.client.api_mock = self.client.vaults

    def test_default(self):
        arglist = [
            '--id', 'vault-id',
            '--name', 'vault-name',
            '--cloud-type', 'cloud-type',
            '--limit', '12',
            '--object-type', 'object-type',
            '--offset', '1',
            '--policy-id', 'policy-id',
            '--protect-type', 'protect-type',
            '--status', 'status',
        ]

        verifylist = [
            ('id', 'vault-id'),
            ('name', 'vault-name'),
            ('cloud_type', 'cloud-type'),
            ('limit', 12),
            ('object_type', 'object-type'),
            ('offset', 1),
            ('policy_id', 'policy-id'),
            ('protect_type', 'protect-type'),
            ('status', 'status')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            id='vault-id',
            name='vault-name',
            cloud_type='cloud-type',
            limit=12,
            object_type='object-type',
            offset=1,
            policy_id='policy-id',
            protect_type='protect-type',
            status='status'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShowVault(fakes.TestCBR):

    object = fakes.FakeVault.create_one()

    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    flat_data = vault._flatten_vault(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['auto_bind'],
        flat_data['auto_expand'],
        flat_data['backup_policy_id'],
        flat_data['created_at'],
        flat_data['description'],
        flat_data['project_id'],
        flat_data['provider_id'],
        flat_data['user_id'],
        flat_data['status'],
        flat_data['operation_type'],
        flat_data['object_type'],
        flat_data['spec_code'],
        flat_data['size'],
        flat_data['consistent_level'],
        flat_data['charging_mode'],
        flat_data['is_auto_pay'],
        flat_data['is_auto_renew'],
    )

    def setUp(self):
        super(TestShowVault, self).setUp()

        self.cmd = vault.ShowVault(self.app, None)

        self.client.find_vault = mock.Mock()

    def test_default(self):
        arglist = [
            'vault'
        ]
        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_vault.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_vault.assert_called_once_with(
            name_or_id='vault',
            ignore_missing=False,)

        self.data, self.columns = vault._add_resources_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = vault._add_tags_to_vault_obj(
            self.object,
            self.data,
            self.columns,
        )

        self.data, self.columns = vault._add_bind_rules_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDeleteVault(fakes.TestCBR):

    def setUp(self):
        super(TestDeleteVault, self).setUp()

        self.cmd = vault.DeleteVault(self.app, None)

        self.client.delete_vault = mock.Mock()

    def test_delete(self):
        arglist = [
            'p1'
        ]
        verifylist = [
            ('vault', 'p1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.delete_vault.side_effect = [{}]

        # Set the response for find_policy
        self.client.find_vault.side_effect = [
            vaultSDK.Vault(id='p1')
        ]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        delete_calls = [
            mock.call(
                vault='p1',
                ignore_missing=False),
        ]

        find_calls = [
            mock.call(
                name_or_id='p1',
                ignore_missing=False),
        ]

        self.client.delete_vault.assert_has_calls(delete_calls)
        self.client.find_vault.assert_has_calls(find_calls)
        self.assertEqual(1, self.client.delete_vault.call_count)


class TestCreateVault(fakes.TestCBR):

    object = fakes.FakeVault.create_one()

    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    flat_data = vault._flatten_vault(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['auto_bind'],
        flat_data['auto_expand'],
        flat_data['backup_policy_id'],
        flat_data['created_at'],
        flat_data['description'],
        flat_data['project_id'],
        flat_data['provider_id'],
        flat_data['user_id'],
        flat_data['status'],
        flat_data['operation_type'],
        flat_data['object_type'],
        flat_data['spec_code'],
        flat_data['size'],
        flat_data['consistent_level'],
        flat_data['charging_mode'],
        flat_data['is_auto_pay'],
        flat_data['is_auto_renew'],
    )

    def setUp(self):
        super(TestCreateVault, self).setUp()

        self.cmd = vault.CreateVault(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.create_vault = mock.Mock()

    def test_default(self):
        arglist = [
            'vault_name',
            '--consistent-level', 'crash_consistent',
            '--backup-policy', 'id',
            '--object-type', 'disk',
            '--size', '40',
        ]
        verifylist = [
            ('name', 'vault_name'),
            ('consistent_level', 'crash_consistent'),
            ('backup_policy', 'id'),
            ('object_type', 'disk'),
            ('size', 40)
        ]

        # Verify cmd is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.create_vault.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.create_vault.assert_called_once_with(
            resources=[],
            backup_policy_id='id',
            bind_rules={'tags': []},
            billing={
                'cloud_type': 'public',
                'protect_type': 'backup',
                'charging_mode': 'post_paid',
                'consistent_level': 'crash_consistent',
                'object_type': 'disk',
                'size': 40,
                'is_auto_renew': True,
                'is_auto_pay': True
            },
            name='vault_name'
        )

        self.data, self.columns = vault._add_resources_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = vault._add_bind_rules_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = vault._add_tags_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestUpdateVault(fakes.TestCBR):

    object = fakes.FakeVault.create_one()

    columns = (
        'ID',
        'name',
        'auto_bind',
        'auto_expand',
        'backup_policy_id',
        'created_at',
        'description',
        'project_id',
        'provider_id',
        'user_id',
        'status',
        'operation_type',
        'object_type',
        'spec_code',
        'size',
        'consistent_level',
        'charging_mode',
        'is_auto_pay',
        'is_auto_renew',
    )

    flat_data = vault._flatten_vault(object)

    data = (
        flat_data['id'],
        flat_data['name'],
        flat_data['auto_bind'],
        flat_data['auto_expand'],
        flat_data['backup_policy_id'],
        flat_data['created_at'],
        flat_data['description'],
        flat_data['project_id'],
        flat_data['provider_id'],
        flat_data['user_id'],
        flat_data['status'],
        flat_data['operation_type'],
        flat_data['object_type'],
        flat_data['spec_code'],
        flat_data['size'],
        flat_data['consistent_level'],
        flat_data['charging_mode'],
        flat_data['is_auto_pay'],
        flat_data['is_auto_renew'],
    )

    def setUp(self):
        super(TestUpdateVault, self).setUp()

        self.cmd = vault.UpdateVault(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.update_vault = mock.Mock()

    def test_default(self):
        arglist = [
            'vault_id',
            '--name', 'vault_name',
            '--size', '40',
        ]
        verifylist = [
            ('vault', 'vault_id'),
            ('name', 'vault_name'),
            ('size', 40),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.update_vault.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_vault.assert_called_with(
            name_or_id='vault_id',
            ignore_missing=False)

        self.client.update_vault.assert_called_once_with(
            vault=mock.ANY,
            billing={
                'size': 40,
            },
            name='vault_name'
        )

        self.data, self.columns = vault._add_resources_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = vault._add_bind_rules_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.data, self.columns = vault._add_tags_to_vault_obj(
            self.object,
            self.data,
            self.columns
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestDissociateVaultResource(fakes.TestCBR):

    def setUp(self):
        super(TestDissociateVaultResource, self).setUp()

        self.cmd = vault.DissociateVaultResource(self.app, None)

        self.client.dissociate_resources = mock.Mock()

    def test_delete(self):
        arglist = [
            'vault',
            '--resource', 'resource'
        ]
        verifylist = [
            ('vault', 'vault'),
            ('resource', ['resource']),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response for find_vault
        self.client.find_vault.side_effect = [
            vaultSDK.Vault(id='vault')
        ]

        # Set the response
        self.client.dissociate_resources.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        dissociate_calls = [
            mock.call(
                vault='vault',
                resources=['resource']),
        ]

        find_calls = [
            mock.call(
                name_or_id='vault',
                ignore_missing=False),
        ]

        self.client.find_vault.assert_has_calls(find_calls)
        self.client.dissociate_resources.assert_has_calls(dissociate_calls)
        self.assertEqual(1, self.client.dissociate_resources.call_count)


class TestUnbindVaultPolicy(fakes.TestCBR):

    def setUp(self):
        super(TestUnbindVaultPolicy, self).setUp()

        self.cmd = vault.UnbindVaultPolicy(self.app, None)

        self.client.unbind_policy = mock.Mock()

    def test_delete(self):
        arglist = [
            'vault',
            'policy'
        ]
        verifylist = [
            ('vault', 'vault'),
            ('policy', 'policy'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response for find_vault
        self.client.find_vault.side_effect = [
            vaultSDK.Vault(id='vault')
        ]

        # Set the response
        self.client.unbind_policy.side_effect = [{}]

        # Trigger the action
        self.cmd.take_action(parsed_args)

        unbind_calls = [
            mock.call(
                vault='vault',
                policy='policy'),
        ]

        find_calls = [
            mock.call(
                name_or_id='vault',
                ignore_missing=False),
        ]

        self.client.find_vault.assert_has_calls(find_calls)
        self.client.unbind_policy.assert_has_calls(unbind_calls)
        self.assertEqual(1, self.client.unbind_policy.call_count)


class TestAssociateVaultResource(fakes.TestCBR):
    object = fakes.VaultDefaultStruct(
        **{
            '_content': b'{"add_resource_ids": ["resource_id"]}'
        }
    )
    columns = (
        'resource_1',
    )
    data = (
        'resource_id',
    )

    def setUp(self):
        super(TestAssociateVaultResource, self).setUp()

        self.cmd = vault.AssociateVaultResource(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.associate_resources = mock.Mock()

    def test_default(self):
        arglist = [
            'vault_id',
            '--resource', 'id=resource_id,type=resource_type'
        ]
        verifylist = [
            ('vault', 'vault_id'),
            ('resource', [{'id': 'resource_id', 'type': 'resource_type'}]),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response for find_vault
        self.client.find_vault.side_effect = [
            vaultSDK.Vault(id='vault_id')
        ]

        # Set the response
        self.client.associate_resources.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.associate_resources.assert_called_with(
            vault='vault_id',
            resources=[{'id': 'resource_id', 'type': 'resource_type'}]
        )

        self.client.associate_resources.assert_called_once_with(
            vault='vault_id',
            resources=[{'id': 'resource_id', 'type': 'resource_type'}]
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestBindVaultPolicy(fakes.TestCBR):
    object = fakes.VaultDefaultStruct(
        **{
            '_content': b'{"associate_policy": '
                        b'{"vault_id" : "vault_id",'
                        b'"policy_id" : "policy_id"}}'
        }
    )
    columns = (
        'vault_id',
        'policy_id',
    )
    data = (
        'vault_id',
        'policy_id',
    )

    def setUp(self):
        super(TestBindVaultPolicy, self).setUp()

        self.cmd = vault.BindVaultPolicy(self.app, None)
        self.app.client_manager.sdk_connection = mock.Mock()

        self.client.bind_policy = mock.Mock()

    def test_default(self):
        arglist = [
            'vault_id',
            'policy_id',
        ]
        verifylist = [
            ('vault', 'vault_id'),
            ('policy', 'policy_id'),
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response for find_vault
        self.client.find_vault.side_effect = [
            vaultSDK.Vault(id='vault_id')
        ]

        # Set the response
        self.client.bind_policy.side_effect = [
            self.object
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.bind_policy.assert_called_with(
            vault='vault_id',
            policy='policy_id'
        )

        self.client.bind_policy.assert_called_once_with(
            vault='vault_id',
            policy='policy_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import exceptions
from openstack import proxy

from otcextensions.sdk.dms.v1 import az as _az
from otcextensions.sdk.dms.v1 import group as _group
from otcextensions.sdk.dms.v1 import instance as _instance
from otcextensions.sdk.dms.v1 import maintenance_window as _mw
from otcextensions.sdk.dms.v1 import message as _message
from otcextensions.sdk.dms.v1 import product as _product
from otcextensions.sdk.dms.v1 import queue as _queue
from otcextensions.sdk.dms.v1 import topic as _topic
from openstack import resource


class Proxy(proxy.Proxy):

    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            'Content-Type': 'application/json',
        }

    # ======== Queues ========
    def create_queue(self, **kwargs):
        """Create a queue

        :param dict kwargs: Keyword arguments which will be used to overwrite a
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        """
        return self._create(_queue.Queue, **kwargs)

    def queues(self, **kwargs):
        """List all queues

        :param dict kwargs: List of query parameters

        :returns: A generator of Queue object of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        """
        return self._list(_queue.Queue, paginated=False, **kwargs)

    def find_queue(self, name_or_id):
        """Find queue by name or id

        :param name_or_id: Name or ID
        :returns: one object of class
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        """
        return self._find(_queue.Queue, name_or_id)

    def get_queue(self, queue):
        """Get detail about a given queue id

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :returns: one object of class
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        """
        return self._get(_queue.Queue, queue)

    def delete_queue(self, queue, ignore_missing=True):
        """Delete queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the queue does not exist.
        :returns: `None`
        """

        self._delete(_queue.Queue, queue, ignore_missing=ignore_missing)

    # ======== Groups ========
    def create_group(self, queue, name):
        """Create a list consume groups for a queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param str name: Group name to create
        :returns: A list of object
            :class:`~otcextensions.sdk.dms.v1.queue.Group`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)

        return self._create(_group.Group, queue_id=queue_obj.id, name=name)

    def groups(self, queue, **kwargs):
        """List all groups for a given queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param dict kwargs: Query parameters

        :returns: A generator of Group object
        :rtype: :class:`~otcextensions.sdk.dms.v1.queue.Group`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)

        return self._list(_group.Group,
                          queue_id=queue_obj.id,
                          paginated=False,
                          **kwargs)

    def find_group(self, queue, name_or_id, ignore_missing=False):
        """Find group by name or id

        :param queue: Queue name or object
        :param name_or_id: Name or ID
        :param ignore_missing:
        :returns: one object of class
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)
        return self._find(_group.Group, name_or_id,
                          ignore_missing=ignore_missing,
                          queue_id=queue_obj.id)

    def delete_group(self, queue, group, ignore_missing=True):
        """Delete a consume on the queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param group: The consume group id or an instance of
            :class:`~otcextensions.sdk.dms.v1.group.Group`
        :returns: ``None``
        """
        queue_obj = self._get_resource(_queue.Queue, queue)

        self._delete(_group.Group, group, queue_id=queue_obj.id,
                     ignore_missing=ignore_missing)

    # ======== Messages ========
    def send_messages(self, queue, messages, return_id=False, **kwargs):
        """Send messages for a given queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param list messages: A list of message dictionaries
        :returns: Messages holder instance
            :class:`otcextensions.sdk.dms.v1.message.Messages`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)
        messages_list = list()
        for msg in messages:
            if not isinstance(msg, _message.Message):
                obj = _message.Message(**msg)
            else:
                obj = msg
            messages_list.append(obj.to_dict(computed=False, ignore_none=True))

        return self._create(_message.Messages,
                            base_path='/queues/%(queue_id)s/messages',
                            queue_id=queue_obj.id,
                            messages=messages_list,
                            return_id=return_id)

    def send_message(self, queue, return_id=True, body=None, **attrs):
        """Send single message into a given queue

        :param queue: The queue id or an instance of
            :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param bool return_id: Whether response should contain message id
        :param body: A str/json object representing message body
        :param dict attr: Additional message attributes
        :returns: Messages holder instance
            :class:`otcextensions.sdk.dms.v1.message.Message`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)
        msg = _message.Message(body=body, **attrs)

        return self._create(_message.Messages,
                            base_path='/queues/%(queue_id)s/messages',
                            queue_id=queue_obj.id,
                            messages=[msg.to_dict(computed=False,
                                                  ignore_none=True)],
                            return_id=return_id).messages[0]

    def consume_message(self, queue, group, **query):
        """Consume queue's message

        :param queue: The queue id or an instance of
          :class:`~otcextensions.sdk.dms.v1.queue.Queue`
        :param group: The consume group id or an instance of
          :class:`~otcextensions.sdk.dms.v1.group.Group`
        :param kwargs query: Optional query parameters to be sent to limit
          the resources being returned.
        :returns: A list of object
          :class:`~otcextensions.sdk.dms.v1.group_message.GroupMessage`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)
        group_obj = self._get_resource(_group.Group, group)

        return self._list(
            _message.Message,
            base_path='/queues/%(queue_id)s/groups/%(group_id)s/messages',
            queue_id=queue_obj.id,
            group_id=group_obj.id,
            **query)

    def ack_message(self, queue, group, messages, status='success'):
        """Confirm consumed message

        :param queue: An queue object
        :param group: A Queue group object
        :param messages: List of messages to be ACKed of
          :class:`~otcextensions.sdk.dms.v1.message.Messages
        :param status: The expeced status of the consumed message
        :returns: An object of an instance of
          :class:`~otcextensions.sdk.dms.v1.group_message.GroupMessage`
        """
        queue_obj = self._get_resource(_queue.Queue, queue)
        group_obj = self._get_resource(_group.Group, group)

        return group_obj.ack(self, queue_obj, messages, status=status)

    # ======== Instances =======
    def instances(self, **kwargs):
        """List all DMS Instances

        :param dict kwargs: List of query parameters

        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        """
        return self._list(_instance.Instance, paginated=False, **kwargs)

    def create_instance(self, **attrs):
        """Create an DMS instance

        :param dict attrs: instance attributes
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        :returns: An instance class object
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        """
        return self._create(_instance.Instance, **attrs)

    def delete_instance(self, instance, ignore_missing=True):
        """Delete DMS Instance

        :param instance: The instance id or an object instance of
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
        :returns: `None`
        """
        self._delete(_instance.Instance, instance,
                     ignore_missing=ignore_missing)

    def find_instance(self, name_or_id, ignore_missing=False):
        """Find DMS Instance by name or id

        :param name_or_id: Name or ID
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.

        :returns: one object of class
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        """
        return self._find(_instance.Instance, name_or_id,
                          ignore_missing=ignore_missing)

    def get_instance(self, instance):
        """Get detail about a given instance id

        :param instance: The instance id or an instance of
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        :returns: one object of class
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        """
        return self._get(_instance.Instance, instance)

    def update_instance(self, instance, **attrs):
        """Update an Instance

        :param instance: Either the ID of an instance or a
            :class:`~otcextensions.sdk.dms.v1.instance.Instance` instance.
        :param dict attrs: The attributes to update on the instance
            represented by ``value``.

        :returns: The updated instance
        :rtype: :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        """
        return self._update(_instance.Instance, instance,
                            **attrs)

    def restart_instance(self, instance):
        """Restart instance

        :param instance: Either the ID of an instance or a
            :class:`~otcextensions.sdk.dms.v1.instance.Instance` instance.
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.restart(self)

    def restart_instances(self, instances_list):
        """Restart multiple instances
        """
        dummy_instance = _instance.Instance()
        return dummy_instance.restart_batch(self, instances_list)

    def delete_batch(self, instances_list):
        """Delete multiple instances
        """
        dummy_instance = _instance.Instance()
        return dummy_instance.delete_batch(self, instances_list)

    def delete_failed(self):
        """Delete failed Kafka instances
        """
        dummy_instance = _instance.Instance()
        return dummy_instance.delete_failed(self)

    # ======== Topics =======
    def topics(self, instance):
        """List all DMS Instance topics

        :param instance: Either the ID of an instance or a
            :class:`~otcextensions.sdk.dms.v1.instance.Instance` instance.
        :param dict kwargs: List of query parameters

        :returns: A generator of Instance object of
            :class:`~otcextensions.sdk.dms.v1.topic.Topic`
        """
        instance_obj = self._get_resource(_instance.Instance, instance)
        return self._list(_topic.Topic,
                          instance_id=instance_obj.id
                          )

    def create_topic(self, instance, **attrs):
        """Create a topic on DMS Instance

        :param instance: instance id or
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        :param dict attrs: Attributes of the topic
            :class:`otcextensions.sdk.dms.v1.topic.Topic`
        :returns: An topic class object
            :class:`~otcextensions.sdk.dms.v1.topic.Topic`
        """
        instance_obj = self._get_resource(_instance.Instance, instance)
        return self._create(_topic.Topic,
                            instance_id=instance_obj.id,
                            **attrs)

    def delete_topic(self, instance, topics, ignore_missing=True):
        """Delete topic on DMS instance

        :param instance: The instance id or an object instance of
            :class:`~otcextensions.sdk.dms.v1.instance.Instance`
        :param list topics: List of topic IDs
        :param bool ignore_missing: When set to ``False``
            :class:`~otcextensions.sdk.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
        :returns: `None`
        """
        instance_obj = self._get_resource(_instance.Instance, instance)

        topics_list = []
        if isinstance(topics, str):
            topics_list.append(topics)
        elif isinstance(topics, list):
            for i in topics:
                if isinstance(i, str):
                    topics_list.append(i)
                elif isinstance(i, _topic.Topic):
                    topics_list.append(i.id)

        response = self.post(
            '/instances/%s/topics/delete' % (instance_obj.id),
            json={'topics': topics_list}
        )
        exceptions.raise_from_response(response)

    # ======== Misc =======
    def availability_zones(self, **kwargs):
        """List all supported DMS Instance availability zones

        :returns: A generator of Instance object of AvailabilityZone
            :class:`~otcextensions.sdk.dms.v1.az.AvailabilityZone`
        """
        return self._list(_az.AvailabilityZone, **kwargs)

    def products(self, **kwargs):
        """List all supported DMS products

        :returns: A generator of Product object
            :class:`~otcextensions.sdk.dms.v1.product.Product`
        """
        return self._list(_product.Product, **kwargs)

    def maintenance_windows(self, **kwargs):
        """List all DMS maintenance windows

        :returns: A generator of MaintenanceWindow object
            :class:`~otcextensions.sdk.dms.v1.maintenance_window.MaintenanceWindow`
        """
        return self._list(_mw.MaintenanceWindow, **kwargs)

    def wait_for_delete(self, res, interval=2, wait=120, callback=None):
        """Wait for a resource to be deleted.

        :param res: The resource to wait on to be deleted.
        :type resource: A :class:`~openstack.resource.Resource` object.
        :param interval: Number of seconds to wait before to consecutive
            checks. Default to 2.
        :param wait: Maximum number of seconds to wait before the change.
            Default to 120.
        :param callback: A callback function. This will be called with a single
            value, progress, which is a percentage value from 0-100.

        :returns: The resource is returned on success.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` if transition
            to delete failed to occur in the specified seconds.
        """
        return resource.wait_for_delete(self, res, interval, wait, callback)

    def _get_cleanup_dependencies(self):
        return {
            'rds': {
                'before': ['network']
            }
        }

    def _service_cleanup(
            self,
            dry_run=True,
            client_status_queue=None,
            identified_resources=None,
            filters=None,
            resource_evaluation_fn=None,
            skip_resources=None,
    ):
        if self.should_skip_resource_cleanup("instance", skip_resources):
            return

        instances = []

        for instance in self.instances():
            for topic in self.topics(instance):
                self._service_cleanup_del_res(
                    self.delete_topic,
                    topic,
                    dry_run=dry_run,
                    client_status_queue=client_status_queue,
                    identified_resources=identified_resources,
                    filters=filters,
                    resource_evaluation_fn=resource_evaluation_fn,
                )
            need_delete = self._service_cleanup_del_res(
                self.delete_instance,
                instance,
                dry_run=dry_run,
                client_status_queue=client_status_queue,
                identified_resources=identified_resources,
                filters=filters,
                resource_evaluation_fn=resource_evaluation_fn,
            )
            if not dry_run and need_delete:
                instances.append(instance)

        for instance in instances:
            self.wait_for_delete(instance)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

from unittest import mock

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import group

GROUP_EXAMPLE = {
    "queue_name": "queue_001",
    "groups": [{
               "id": "g-5ec247fd-d4a2-4d4f-9876-e4ff3280c461",
               "name": "abcDffD",
               "produced_messages": 0,
               "consumed_messages": 0,
               "available_messages": 0
               }],
    "redrive_policy": "enable",
    "queue_id": "9bf46390-38a2-462d-b392-4d5b2d519c55"
}


class TestGroup(base.TestCase):

    example = GROUP_EXAMPLE['groups'][0]
    objcls = group.Group

    def setUp(self):
        super(TestGroup, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.default_microversion = None

    def test_basic(self):
        sot = self.objcls()

        self.assertEqual('queues/%(queue_id)s/groups', sot.base_path)

        self.assertDictEqual({
            'current_page': 'current_page',
            'include_deadletter': 'include_deadletter',
            'limit': 'limit',
            'marker': 'marker',
            'page_size': 'page_size'},
            sot._query_mapping._mapping
        )

    def test_make_it(self):

        sot = self.objcls(**self.example)
        self.assertEqual(self.example['id'], sot.id)
        self.assertEqual(self.example['name'], sot.name)
        self.assertEqual(
            self.example['produced_messages'],
            sot.produced_messages)
        self.assertEqual(
            self.example['consumed_messages'],
            sot.consumed_messages)
        self.assertEqual(
            self.example['available_messages'],
            sot.available_messages)

    def test_create(self):
        sot = group.Group(queue_id='qid', name='grp')

        response = mock.Mock()
        response.body = {'groups': [{'id': '1', 'name': 'grp'}]}
        response.json = mock.Mock(return_value=response.body)
        self.sess.post = mock.Mock(return_value=response)

        obj = sot.create(self.sess)
        self.sess.post.assert_called_with(
            'queues/qid/groups',
            json={'groups': [{'name': 'grp'}]}
        )
        self.assertEqual('grp', obj.name)
        self.assertEqual('1', obj.id)

    def test_ack(self):
        sot = group.Group(queue_id='qid', id='gid')

        response = mock.Mock()
        response.json = mock.Mock(return_value={})
        response.status_code = 200
        self.sess.post = mock.Mock(return_value=response)

        sot.ack(self.sess, mock.Mock(id='qid'), ['1', '2'], 'failure')

        self.sess.post.assert_called_with(
            'queues/qid/groups/gid/ack',
            json={'message': [
                {'handler': '1', 'status': 'failure'},
                {'handler': '2', 'status': 'failure'}
            ]}
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
"""ModelArts training job v1 action implementations"""
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    hidden = ["location"]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListBuiltInModels(command.Lister):
    _description = _("Get properties of a vm")
    columns = (
        "Model Id",
        "Model name",
        "Model Usage",
        "Model Precision",
        "Model Size",
        "Created At",
    )

    def get_parser(self, prog_name):
        parser = super(ListBuiltInModels, self).get_parser(prog_name)

        sort_by_choices = [
            "create_time",
            "engine",
            "model_name",
            "model_precision",
            "model_usage",
            "model_precision",
            "model_size" "parameter",
        ]

        parser.add_argument(
            "--limit",
            metavar="<limit>",
            type=int,
            help=_(
                "Number of built-in algorithms displayed on each page. "
                "The value range is [1, 100]. Default value: 10."
            ),
        )
        parser.add_argument(
            "--offset",
            metavar="<offset>",
            type=int,
            help=_("Index of the page to be queried. Default value: 1"),
        )
        parser.add_argument(
            "--sort-by",
            metavar="{" + ",".join(sort_by_choices) + "}",
            type=lambda s: s.lower(),
            choices=sort_by_choices,
            help=_("Sorting field. Default value: job_name"),
        )
        parser.add_argument(
            "--order",
            metavar="{asc, desc}",
            type=lambda s: s.lower(),
            choices=["asc", "desc"],
            help=_("Sorting order. Default value: desc"),
        )
        parser.add_argument(
            "--search-content",
            metavar="<search_content>",
            help=_("Search content, for example, a parameter name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1

        attrs_list = (
            "limit",
            "offset",
            "sort_by",
            "order",
            "search_content",
        )
        query_params = {}
        for arg in attrs_list:
            val = getattr(parsed_args, arg)
            if val or str(val) == "0":
                query_params[arg] = val

        data = client.builtin_models(**query_params)
        _formatters = {
            "Created At": cli_utils.UnixTimestampFormatter,
        }
        table = (
            self.columns,
            (
                utils.get_dict_properties(
                    s, self.columns, formatters=_formatters
                )
                for s in data
            ),
        )
        return table


class ShowBuiltInModel(command.ShowOne):
    _description = _("Show details of a built-in model")

    def get_parser(self, prog_name):
        parser = super(ShowBuiltInModel, self).get_parser(prog_name)
        parser.add_argument(
            "name",
            metavar="<name>",
            help=_("Enter model built-in model name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.modelartsv1
        data = client.find_builtin_model(
            parsed_args.name, ignore_missing=False
        )

        display_columns, columns = _get_columns(data)

        formatters = {
            "created_at": cli_utils.UnixTimestampFormatter,
            "parameter": cli_utils.YamlFormat,
        }
        data = utils.get_item_properties(data, columns, formatters=formatters)
        return (display_columns, data)



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a training job version from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")
job_id = 8742
attrs = {
    "job_desc": "This is a ModelArts job",
    "config": {
        "worker_server_num": 1,
        "app_url": "/usr/app/",
        "boot_file_url": "/usr/app/boot.py",
        "parameter": [
            {
                "label": "learning_rate",
                "value": "0.01",
            },
            {
                "label": "batch_size",
                "value": "32",
            },
        ],
        "dataset_id": "38277e62-9e59-48f4-8d89-c8cf41622c24",
        "dataset_version_id": "2ff0d6ba-c480-45ae-be41-09a8369bfc90",
        "spec_id": 1,
        "engine_id": 1,
        "train_url": "/usr/train/",
        "log_url": "/usr/log/",
        "pre_version_id": 20,
    },
}

trainingjob_version = conn.modelartsv1.create_trainingjob_version(
    job_id, **attrs
)
print(trainingjob_version)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import eip

EXAMPLE = {
    "job_id": "3711e2ad-5787-49bc-a47f-3f0b066af9f5",
    "node_id": "52a4c096bb1f455d8d866956a959519eno02",
    "node_name": "mongodb-8977_mongos_node_1",
    "public_ip": "10.145.51.128",
    "public_ip_id": "45da4782-e0c8-4aa4-a290-b8740014f710"
}


class TestEip(base.TestCase):
    def test_basic(self):
        sot = eip.Eip()
        path = '/nodes'
        self.assertEqual(path, sot.base_path)

    def test_make_it(self):
        sot = eip.Eip(**EXAMPLE)
        self.assertEqual(EXAMPLE['job_id'], sot.job_id)
        self.assertEqual(EXAMPLE['node_id'], sot.node_id)
        self.assertEqual(EXAMPLE['node_name'], sot.node_name)
        self.assertEqual(EXAMPLE['public_ip'], sot.public_ip)
        self.assertEqual(EXAMPLE['public_ip_id'], sot.public_ip_id)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.sdk.dcs.v1 import restore_record

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
FAKE_INSTANCE_ID = "some_fake_id"
EXAMPLE = {
    "status": "succeed",
    "progress": "100.00",
    "restore_id": "a6155972-800c-4170-a479-3231e907d2f6",
    "backup_id": "f4823e9e-fe9b-4ffd-be79-4e5d6de272bb",
    "restore_remark": "doctest",
    "backup_remark": None,
    "created_at": "2017-07-18T21:41:20.721Z",
    "updated_at": "2017-07-18T21:41:35.182Z",
    "restore_name": "restore_20170718214120",
    "backup_name": "backup_20170718000002",
    "error_code": None
}


class TestRestoreRecord(base.TestCase):

    def test_basic(self):
        sot = restore_record.RestoreRecord()

        self.assertEqual('/instances/%(instance_id)s/restores', sot.base_path)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = restore_record.RestoreRecord(
            instance_id=FAKE_INSTANCE_ID,
            **EXAMPLE)
        self.assertEqual(FAKE_INSTANCE_ID, sot.instance_id)
        self.assertEqual(EXAMPLE['restore_id'], sot.id)
        self.assertEqual(EXAMPLE['backup_id'], sot.backup_id)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['progress'], sot.progress)
        self.assertEqual(EXAMPLE['restore_remark'], sot.restore_description)
        self.assertEqual(EXAMPLE['backup_remark'], sot.backup_description)
        self.assertEqual(EXAMPLE['restore_name'], sot.restore_name)
        self.assertEqual(EXAMPLE['backup_name'], sot.backup_name)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['error_code'], sot.error_code)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import _log
from openstack import resource

from otcextensions.sdk import sdk_resource
from otcextensions.sdk.mrs.v1 import _base
_logger = _log.setup_logging('openstack')


class ParametersSpec (resource.Resource):
    #: Properties
    #: Order ID obtained by the system during scale-out or scale-in.
    order_id = resource.Body('order_id')
    #: Cluster scale-in or scale-out type.
    scale_type = resource.Body('scale_type')
    #: ID of the newly added or removed node.
    node_id = resource.Body('node_id')
    #: Node group to be scaled out or in.
    node_group = resource.Body('node_group')
    #: Task node specifications.
    task_node_info = resource.Body('task_node_info', type=dict)
    #: Number of nodes to be added or removed.
    instances = resource.Body('instances', type=int)
    #: Indicates whether the bootstrap action specified
    #: during cluster creation is performed on nodes.
    skip_bootstrap_scripts = resource.Body('skip_bootstrap_scripts', type=bool)
    #: Whether to start components on the added nodes after cluster scale-out.
    scale_without_start = resource.Body('scale_without_start', type=bool)
    #: ID list of Task nodes to be deleted during task node scale-in..
    server_ids = resource.Body('server_ids', type=list)


class ClusterInfo(_base.Resource):
    resource_key = 'cluster'
    resources_key = 'clusters'
    base_path = '/cluster_infos'

    # capabilities
    allow_list = True
    allow_get = True
    allow_update = True

    _query_mapping = resource.QueryParameters(
        'status', 'marker', 'limit',
        'project_id', 'tags',
        status='clusterState',
        marker='currentPage',
        limit='pageSize')

    cluster_id = resource.URI('cluster_id')

    #: Properties
    #: Cluster ID.
    id = resource.Body('clusterId', alternate_id=True)
    #: Cluster name.
    name = resource.Body('clusterName')
    #: Number of Master nodes deployed in a cluster.
    master_num = resource.Body('masterNodeNum')
    #: Number of Core nodes deployed in a cluster.
    core_num = resource.Body('coreNodeNum')
    #: Total number of nodes deployed in a cluster.
    total_num = resource.Body('totalNodeNum')
    #: Cluster status.
    status = resource.Body('clusterState')
    #: Cluster creation time, which is a 10-bit timestamp.
    created_at = resource.Body('createAt')
    #: Cluster update time, which is a 10-bit timestamp.
    updated_at = resource.Body('updateAt')
    #: Cluster billing mode
    billing_type = resource.Body('billingType')
    #: Cluster work region.
    region = resource.Body('dataCenter')
    #: VPC name.
    vpc = resource.Body('vpc')
    #: Cluster creation fee, which is automatically calculated.
    fee = resource.Body('fee')
    #: Hadoop version.
    hadoop_version = resource.Body('hadoopVersion')
    #: Instance specifications of a Master node.
    master_node_size = resource.Body('masterNodeSize')
    #: Instance specifications of a Core node.
    core_node_size = resource.Body('coreNodeSize')
    #: Component list.
    component_list = resource.Body('componentList', type=list)
    #: External IP address.
    external_ip = resource.Body('externalIp')
    #: Backup external IP address.
    external_alternate_ip = resource.Body('externalAlternateIp')
    #: Internal IP address.
    internal_ip = resource.Body('internalIp')
    #: Cluster deployment ID
    deployment_id = resource.Body('deploymentId')
    #: Cluster remarks.
    remark = resource.Body('remark')
    #: Cluster creation order ID.
    order_id = resource.Body('orderId')
    #: AZ name.
    az = resource.Body('azName')
    #: AZ ID.
    az_id = resource.Body('azId')
    #: Product ID of a Master node.
    master_node_product_id = resource.Body('masterNodeProductId')
    #: Specification ID of a Master node.
    master_node_spec_id = resource.Body('masterNodeSpecId')
    #: Product ID of a Core node.
    core_node_product_id = resource.Body('coreNodeProductId')
    #: Specification ID of a Core node.
    core_node_spec_id = resource.Body('coreNodeSpecId')
    #: Instance ID.
    instance_id = resource.Body('instanceId')
    #: URI for remotely logging in to an ECS.
    vnc = resource.Body('vnc')
    #: Project ID.
    project_id = resource.Body('tenantId')
    #: Disk storage space.
    volume_size = resource.Body('volumeSize', type=int)
    #: Disk type.
    volume_type = resource.Body('volumeType')
    #: Subnet ID.
    subnet_id = resource.Body('subnetId')
    #: Subnet name.
    subnet_name = resource.Body('subnetName')
    #: Cluster type.
    cluster_type = resource.Body('clusterType')
    #: Security group ID.
    security_group_id = resource.Body('securityGroupsId')
    #: Security group ID of a non-Master node.
    non_master_security_group_id = resource.Body('slaveSecurityGroupsId')
    #: Cluster operation progress description.
    stage_desc = resource.Body('stageDesc')
    #: Whether MRS Manager installation is finished during cluster creation.
    mrs_install_state = resource.Body('mrsManagerFinish', type=bool)
    #: Running mode of an MRS cluster.
    safe_mode = resource.Body('safeMode')
    #: Cluster version.
    cluster_version = resource.Body('clusterVersion')
    #: Name of the key file.
    key = resource.Body('nodePublicCertName')
    #: IP address of a Master node.
    master_ip = resource.Body('masterNodeIp')
    #: Preferred private IP address.
    preffered_private_ip = resource.Body('privateIpFirst')
    #: Error message.
    error_info = resource.Body('errorInfo')
    #: Start time of billing.
    charging_start_time = resource.Body('chargingStartTime')
    #: Whether to collect logs when cluster installation fails.
    log_collection = resource.Body('logCollection', type=int)
    #: List of Task nodes.
    task_node_groups = resource.Body('taskNodeGroups', type=list)
    #: List of Master, Core and Task nodes.
    node_groups = resource.Body('nodeGroups', type=list)
    #: Data disk storage type of the Master node.
    #: SATA, SAS and SSD are supported.
    master_data_volume_type = resource.Body('masterDataVolumeType')
    #: Data disk storage space of the Master node.
    master_data_volume_size = resource.Body('masterDataVolumeSize', type=int)
    #: Number of data disks of the Master node.
    master_data_volume_count = resource.Body('masterDataVolumeCount', type=int)
    #: Data disk storage type of the Core node.
    core_data_volume_type = resource.Body('coreDataVolumeType')
    #: Data disk storage space of the Core node.
    core_data_volume_size = resource.Body('coreDataVolumeSize', type=int)
    #: Data disk storage space of the Core node.
    core_data_volume_count = resource.Body('coreDataVolumeCount', type=int)
    #: Bootstrap action script information.
    bootstrap_scripts = resource.Body('bootstrap_scripts', type=list)
    #: Bootstrap action script information.
    tags = resource.Body('tags', type=list)
    #: Node change status.
    scale = resource.Body('scale')
    #: Core parameters.
    parameters = resource.Body('parameters', type=ParametersSpec)


class Host(sdk_resource.Resource):
    resources_key = 'hosts'
    base_path = '/clusters/%(cluster_id)s/hosts'

    # capabilities
    allow_list = True
    allow_get = True

    _query_mapping = resource.QueryParameters(
        'marker', 'limit',
        marker='currentPage',
        limit='pageSize')

    cluster_id = resource.URI('cluster_id')

    #: Properties
    #: VM ID
    id = resource.Body('id', alternate_id=True)
    #: VM IP address
    ip = resource.Body('ip')
    #: VM flavor ID
    flavor = resource.Body('flavor')
    #: VM type
    type = resource.Body('type')
    #: VM name
    name = resource.Body('name')
    #: Current VM state
    status = resource.Body('status')
    #: Memory
    mem = resource.Body('mem')
    #: Number of CPU cores
    cpu = resource.Body('cpu')
    #: OS disk capacity
    root_volume_size = resource.Body('root_volume_size')
    #: Data disk type
    data_volume_type = resource.Body('data_volume_type')
    #: Data disk capacity
    data_volume_size = resource.Body('data_volume_size')
    #: Number of data disks
    data_volume_count = resource.Body('data_volume_count')


class Cluster(sdk_resource.Resource):
    base_path = '/clusters'

    # capabilities
    allow_delete = True



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import datetime
import random
import uuid

import mock

from openstackclient.tests.unit import utils

from otcextensions.tests.unit.osclient import test_base

from otcextensions.sdk.auto_scaling.v1 import activity
from otcextensions.sdk.auto_scaling.v1 import config
from otcextensions.sdk.auto_scaling.v1 import group
from otcextensions.sdk.auto_scaling.v1 import instance
from otcextensions.sdk.auto_scaling.v1 import policy
from otcextensions.sdk.auto_scaling.v1 import quota


class TestAutoScaling(utils.TestCommand):

    def setUp(self):
        super(TestAutoScaling, self).setUp()

        self.app.client_manager.auto_scaling = mock.Mock()


class FakeGroup(test_base.Fake):
    """Fake one or more Group"""

    @classmethod
    def generate(cls):
        object_info = {
            'create_time': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'name': 'group-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'status': 'SOME STATUS',
            'detail': 'detail-' + uuid.uuid4().hex,
            'router_id': 'id-vpc-' + uuid.uuid4().hex,
        }
        obj = group.Group.existing(**object_info)
        return obj


class FakeConfig(test_base.Fake):
    """Fake one or more AS Config"""

    @classmethod
    def generate(cls):
        object_info = {
            'create_time': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'name': 'name-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'instance_config': {
                'instance_id': 'inst_id-' + uuid.uuid4().hex,
                'instance_name': 'inst-name-' + uuid.uuid4().hex,
                'flavor_id': 'flavor-' + uuid.uuid4().hex,
                'image_id': 'image-' + uuid.uuid4().hex,
                'disk': [
                    {
                        'size': random.randint(1, 200),
                        'volume_type': 'SSD',
                        'disk_type': 'SYS'
                    }
                ],
                'public_ip': None,
                'user_data': None,
                'metadata': {},
            }
        }
        obj = config.Config.existing(**object_info)
        return obj


class FakePolicy(test_base.Fake):
    """Fake one or more AS Policy"""

    @classmethod
    def generate(cls):
        object_info = {
            'create_time': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
            'name': 'name-' + uuid.uuid4().hex,
            'id': 'id-' + uuid.uuid4().hex,
            'type': 'type-' + uuid.uuid4().hex,
            'scaling_group_id': 'sgid-' + uuid.uuid4().hex,
            'alarm_id': 'alarmid-' + uuid.uuid4().hex,
            'cool_down_time': random.randint(1, 10000),
            'status': 'status-' + uuid.uuid4().hex,
            'scheduled_policy': {
                'launch_time': 'launch_time-' + uuid.uuid4().hex,
                'recurrence_type': 'recurrence_type-' + uuid.uuid4().hex,
                'recurrence_value': 'recurrence_value-' + uuid.uuid4().hex,
                'start_time': 'start_time-' + uuid.uuid4().hex,
                'end_time': 'end_time-' + uuid.uuid4().hex,
            },
            'scaling_policy_action': {
                'operation': 'operation-' + uuid.uuid4().hex,
                'instance_number': random.randint(1, 100),
            }
        }
        obj = policy.Policy.existing(**object_info)
        return obj


class FakeActivity(test_base.Fake):
    """Fake one or more AS Activity"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'start_time': 'st-' + uuid.uuid4().hex,
            'end_time': 'et-' + uuid.uuid4().hex,
            'description': 'description-' + uuid.uuid4().hex,
            'status': 'status-' + uuid.uuid4().hex,
            'scaling_value': 'sv-' + uuid.uuid4().hex,
            'instance_value': random.randint(1, 10000),
            'desire_value': random.randint(1, 10000),
        }
        obj = activity.Activity.existing(**object_info)
        return obj


class FakeInstance(test_base.Fake):
    """Fake one or more AS Instance"""

    @classmethod
    def generate(cls):
        object_info = {
            'id': 'id-' + uuid.uuid4().hex,
            'name': 'name-' + uuid.uuid4().hex,
            'scaling_group_name': 'sgn-' + uuid.uuid4().hex,
            'scaling_configuration_id': 'sci-' + uuid.uuid4().hex,
            'scaling_configuration_name': 'scn-' + uuid.uuid4().hex,
            'lifecycle_state': 'ls-' + uuid.uuid4().hex,
            'health_status': 'hs-' + uuid.uuid4().hex,
            'create_time': datetime.datetime(
                random.randint(2000, 2020),
                random.randint(1, 12),
                random.randint(1, 28)
            ),
        }
        obj = instance.Instance.existing(**object_info)
        return obj


class FakeQuota(test_base.Fake):
    """Fake one or more AS Quota"""

    @classmethod
    def generate(cls):
        object_info = {
            'type': 'id-' + uuid.uuid4().hex,
            'used': random.randint(1, 10000),
            'quota': random.randint(1, 10000),
            'max': random.randint(1, 10000),
        }
        obj = quota.Quota.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base

from otcextensions.common import format


class TestTimeTMsStr(base.TestCase):

    def test_serialize(self):
        obj = format.TimeTMsStr()

        self.assertEqual(
            1534583170000,
            obj.serialize('2018-08-18T09:06:10+00:00')
        )

    def test_deserialize(self):
        obj = format.TimeTMsStr()

        self.assertEqual(
            '2018-08-18T09:06:10.432+00:00',
            obj.deserialize(1534583170432)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from keystoneauth1 import adapter

from unittest import mock

from openstack.tests.unit import base

from otcextensions.sdk.dms.v1 import topic


JSON_DATA = {
    'id': 'haha',
    'partition': 3,
    'replication': 3,
    'sync_replication': True,
    'retention_time': 10,
    'sync_message_flush': True
}


class TestTopic(base.TestCase):

    def setUp(self):
        super(TestTopic, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = topic.Topic()

        self.assertEqual('/instances/%(instance_id)s/topics', sot.base_path)
        self.assertEqual('topics', sot.resources_key)

        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):

        sot = topic.Topic(**JSON_DATA)
        self.assertEqual(JSON_DATA['id'], sot.id)
        self.assertEqual(JSON_DATA['partition'], sot.partition)
        self.assertEqual(JSON_DATA['replication'], sot.replication)
        self.assertEqual(JSON_DATA['retention_time'], sot.retention_time)
        self.assertEqual(JSON_DATA['sync_message_flush'], sot.is_sync_flush)
        self.assertEqual(JSON_DATA['sync_replication'],
                         sot.is_sync_replication)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
try:
    from unittest.mock import patch
except ImportError:
    from mock import patch

from openstack.tests.unit import test_proxy_base

from otcextensions.sdk.cce.v1 import _proxy
from otcextensions.sdk.cce.v1 import cluster as _cluster
from otcextensions.sdk.cce.v1 import cluster_node as _cluster_node


class TestCCEProxy(test_proxy_base.TestProxyBase):
    # HEADERS = {
    #     'Content-Type': 'application/json'
    # }

    def setUp(self):
        super(TestCCEProxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestCCECluster(TestCCEProxy):

    def test_list(self):
        self.verify_list(
            self.proxy.clusters, _cluster.Cluster,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
            method_kwargs={},
            expected_kwargs={
                'paginated': False
            }
        )

    def test_get(self):
        self.verify_get(
            self.proxy.get_cluster,
            _cluster.Cluster,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._get',
            expected_kwargs={}
        )

    def test_create(self):
        self.verify_create(
            self.proxy.create_cluster, _cluster.Cluster,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._create',
            method_kwargs={
                'instance': 'test',
                'name': 'some_name'
            },
            expected_kwargs={
                'prepend_key': False,
                'instance': 'test',
                'name': 'some_name'
            }
        )

    def test_delete(self):
        self.verify_delete(
            self.proxy.delete_cluster,
            _cluster.Cluster, True,
            mock_method='otcextensions.sdk.sdk_proxy.Proxy._delete',
            expected_kwargs={}
        )


class TestCCEClusterNode(TestCCEProxy):

    # @patch('otcextensions.sdk.sdk_proxy.Proxy._find')
    def test_list(self):
        with patch.object(self.proxy, '_find',
                          return_value=_cluster.Cluster(id='cluster_id')):
            self.verify_list(
                self.proxy.cluster_nodes, _cluster_node.ClusterNode,
                mock_method='otcextensions.sdk.sdk_proxy.Proxy._list',
                method_args=['cluster_id'],
                method_kwargs={},
                expected_kwargs={
                    'paginated': False,
                    'cluster_uuid': 'cluster_id'
                },
                expected_args=[]
            )

    def test_get(self):
        with patch.object(self.proxy, '_find',
                          return_value=_cluster.Cluster(id='cluster_id')):
            self._verify(
                'otcextensions.sdk.sdk_proxy.Proxy._get',
                self.proxy.get_cluster_node,
                method_args=['cluster_id', 'node'],
                expected_args=[_cluster_node.ClusterNode, 'node'],
                expected_kwargs={
                    'cluster_uuid': 'cluster_id'
                }
            )

    def test_add_node(self):
        attrs = {'a': 'b'}
        with patch.object(self.proxy, '_find',
                          return_value=_cluster.Cluster(id='cluster_id')):
            self._verify(
                'otcextensions.sdk.cce.v1.cluster_node.ClusterNode.create',
                self.proxy.add_node,
                method_args=['cluster_id'],
                method_kwargs=attrs,
                expected_args=[self.proxy],
                expected_kwargs={
                    'endpoint_override': None,
                    'headers': None,
                    'prepend_key': True,
                    'requests_auth': None
                }
            )

    def test_delete_nodes(self):
        with patch.object(self.proxy, '_find',
                          return_value=_cluster.Cluster(id='cluster_id')):
            self._verify(
                'otcextensions.sdk.cce.v1.cluster.Cluster.delete_nodes',
                self.proxy.delete_cluster_nodes,
                method_args=['cluster_id', ['n1', 'n2']],
                expected_args=[self.proxy, ['n1', 'n2']],
                expected_kwargs={}
            )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""subscription v2 action implementations"""

import logging

from osc_lib import utils
from osc_lib import exceptions
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


def _get_columns(item):
    column_map = {}
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map)


class ListSubscription(command.Lister):

    _description = _("List SMN Subscriptions.")
    columns = (
        'ID',
        'Protocol',
        'Topic URN',
        'Owner',
        'Endpoint',
        'Status'
    )

    def get_parser(self, prog_name):
        parser = super(ListSubscription, self).get_parser(prog_name)

        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_("Resources after this offset will be queried."),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_("Limit to fetch number of records.\n"
                   "Value range: 1–100"),
        )
        parser.add_argument(
            '--topic',
            metavar='<topic>',
            help=_("Specify the topic ID or Name."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        args_list = [
            'offset',
            'limit']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val
        topic = getattr(parsed_args, 'topic') or None
        if topic:
            topic = client.get_topic(topic)

        data = client.subscriptions(topic, **attrs)

        return (self.columns, (utils.get_item_properties(s, self.columns)
                               for s in data))


class CreateSubscription(command.ShowOne):
    _description = _("Add a new Subscription to a SMN topic.")

    def get_parser(self, prog_name):
        parser = super(CreateSubscription, self).get_parser(prog_name)
        parser.add_argument(
            'topic',
            metavar='<topic>',
            help=_("Unique resource ID or name of a topic."),
        )
        parser.add_argument(
            '--endpoint',
            metavar='<endpoint>',
            required=True,
            help=_("Message endpoint."),
        )
        parser.add_argument(
            '--protocol',
            metavar='<protocol>',
            required=True,
            help=_("Subscription protocol.\n"
                   "Currently, the following protocols are supported:\n"
                   "email: The endpoints are email address.\n"
                   "sms: The endpoints are phone numbers.\n"
                   "http and https: The endpoints are URLs."),
        )
        parser.add_argument(
            '--remark',
            metavar='<remark>',
            help=_("Description of the subscription."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn

        args_list = [
            'protocol',
            'endpoint',
            'remark']
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        topic = client.get_topic(parsed_args.topic)
        obj = client.create_subscription(topic, **attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeleteSubscription(command.Command):

    _description = _("Deletes SMN subscription.")

    def get_parser(self, prog_name):
        parser = super(DeleteSubscription, self).get_parser(prog_name)
        parser.add_argument(
            'subscription',
            metavar='<subscription>',
            nargs='+',
            help=_("SMN Subscrition(s) to delete."),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.smn
        result = 0
        for subscription in parsed_args.subscription:
            try:
                client.delete_subscription(subscription, ignore_missing=False)
            except Exception as e:
                result += 1
                LOG.error(_("Failed to delete SMN Subscription with "
                          "ID '%(subscription)s': %(e)s"),
                          {'subscription': subscription, 'e': e})
        if result > 0:
            total = len(parsed_args.subscription)
            msg = (_("%(result)s of %(total)s SMN Subscription(s) failed "
                   "to delete.") % {'result': result, 'total': total})
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from keystoneauth1 import adapter

from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import visualization_job
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples

EXAMPLE = examples.VISUALIZATION_JOB


class TestVisualizationJob(base.TestCase):
    def setUp(self):
        super(TestVisualizationJob, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = visualization_job.VisualizationJob()

        self.assertEqual("/visualization-jobs", sot.base_path)
        self.assertEqual("jobs", sot.resources_key)
        self.assertEqual(None, sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = visualization_job.VisualizationJob(**EXAMPLE)

        for key, value in EXAMPLE.items():
            if key == "create_time":
                self.assertEqual(EXAMPLE["create_time"], sot.created_at)
            else:
                self.assertEqual(getattr(sot, key), value)

    def test_action(self):
        data = {"id": "mock-id"}
        sot = visualization_job.VisualizationJob(**data)
        action = "restart"
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = data
        response.headers = {}
        self.sess.post.return_value = response
        rt = sot._action(self.sess, action)
        self.sess.post.assert_called_with(
            f"visualization-jobs/{sot.id}/{action}",
            json=None,
        )
        self.assertEqual(rt, sot)

    def test_restart(self):
        sot = visualization_job.VisualizationJob.existing(id=EXAMPLE["job_id"])
        sot._action = mock.Mock()
        sot.restart(self.sess)
        sot._action.assert_called_with(self.sess, "restart")

    def test_stop(self):
        sot = visualization_job.VisualizationJob.existing(id=EXAMPLE["job_id"])
        sot._action = mock.Mock()
        sot.stop(self.sess)
        sot._action.assert_called_with(self.sess, "stop")



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource

from otcextensions.sdk.identity.v3 import _bad_base as _base


class Credential(_base.BadBaseResource):
    resource_key = 'credential'
    resources_key = 'credentials'
    base_path = '/v3.0/OS-CREDENTIAL/credentials'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    commit_method = 'PUT'

    _query_mapping = resource.QueryParameters(
        'user_id',
    )

    # Properties
    #: Access key
    access = resource.Body('access', alternate_id=True)
    #: Creation time
    created_at = resource.Body('create_time')
    #: Description
    description = resource.Body('description')
    #: Secret key
    secret = resource.Body('secret')
    #: Status
    status = resource.Body('status')
    #: References the user ID which owns the credential. *Type: string*
    user_id = resource.Body('user_id')



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import exceptions
from openstack import proxy
from otcextensions.sdk.vpcep.v1 import connection as _connection
from otcextensions.sdk.vpcep.v1 import endpoint as _endpoint
from otcextensions.sdk.vpcep.v1 import quota as _quota
from otcextensions.sdk.vpcep.v1 import service as _service
from otcextensions.sdk.vpcep.v1 import public_service as _public_service
from otcextensions.sdk.vpcep.v1 import target_service as _target_service
from otcextensions.sdk.vpcep.v1 import whitelist as _whitelist


class Proxy(proxy.Proxy):
    skip_discovery = True

    # ======== VPC Endpoint ========

    def endpoints(self, **query):
        """Return a generator of endpoints

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:
        :returns: A generator of endpoint objects
        :rtype: :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint`
        """
        if query.get('limit'):
            query.update(paginated=False)
        return self._list(_endpoint.Endpoint, **query)

    def create_endpoint(self, **attrs):
        """Create a new endpoint from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint`,
            comprised of the properties on the Endpoint class.

        :returns: The results of endpoint creation
        :rtype: :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint`
        """
        if attrs.get('ip') and attrs.get('port_ip'):
            raise TypeError(
                "You can use either the 'ip' or the 'port_ip' keyword argument"
            )
        elif attrs.get('ip'):
            attrs['port_ip'] = attrs['ip']
            del attrs['ip']
        return self._create(_endpoint.Endpoint, **attrs)

    def get_endpoint(self, endpoint):
        """Get a single endpoint

        :param endpoint: The value can be the ID of a endpoint or a
            :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint`
            instance.

        :returns: One :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.

        """
        return self._get(_endpoint.Endpoint, endpoint)

    def delete_endpoint(self, endpoint, ignore_missing=True):
        """Delete a endpoint

        :param endpoint: The value can be either the ID of a endpoint or a
            :class:`~otcextensions.sdk.vpcep.endpoint.Endpoint` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the endpoint does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent endpoint.

        :returns: ``None``
        """
        return self._delete(
            _endpoint.Endpoint, endpoint, ignore_missing=ignore_missing
        )

    # ======== Endpoint Service ========

    def services(self, **query):
        """Return a generator of endpoint services

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:

        :returns: A generator of endpoint service objects
        :rtype: :class:`~otcextensions.sdk.vpcep.service.Service`
        """
        if query.get('limit'):
            query.update(paginated=False)
        return self._list(_service.Service, **query)

    def create_service(self, **attrs):
        """Create a new endpoint service from attributes

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.vpcep.service.Service`,
            comprised of the properties on the Service class.

        :returns: The results of endpoint service creation
        :rtype: :class:`~otcextensions.sdk.vpcep.service.Service`
        """
        return self._create(_service.Service, **attrs)

    def get_service(self, service):
        """Get a single endpoint service

        :param service: The value can be the ID of a endpoint service or a
            :class:`~otcextensions.sdk.vpcep.service.Service`
            instance.

        :returns: One :class:`~otcextensions.sdk.vpcep.service.Service`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.

        """
        return self._get(_service.Service, service)

    def find_service(self, name_or_id, ignore_missing=False):
        """Find a single endpoint service

        :param name_or_id: The name or ID of a service.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns: One :class:`~otcextensions.sdk.vpcep.service.Service`
            or None
        """
        return self._find(
            _service.Service,
            name_or_id,
            ignore_missing=ignore_missing,
        )

    def update_service(self, service, **attrs):
        """Update a endpoint service

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param dict attrs: The attributes to update on the endpoint service
            represented by ``service``.

        :returns: The updated service
        :rtype: :class:`~otcextensions.sdk.vpcep.service.Service`
        """
        return self._update(_service.Service, service, **attrs)

    def delete_service(self, service, ignore_missing=True):
        """Delete a endpoint service

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the service does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent service.

        :returns: ``None``
        """
        return self._delete(
            _service.Service,
            service,
            ignore_missing=ignore_missing,
        )

    # ======== Endpoint Service Connections ========

    def service_connections(self, service, **query):
        """Return a generator of service connections

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:

        :returns: A generator of connection objects.
        :rtype: :class:`~otcextensions.sdk.vpcep.connection.Connection`
        """
        if query.get('limit'):
            query.update(paginated=False)
        endpoint_service = self._get_resource(_service.Service, service)
        return self._list(
            _connection.Connection,
            endpoint_service_id=endpoint_service.id,
            **query
        )

    def manage_service_connections(self, service, action, endpoints=[]):
        """Manage endpoint service connections

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param action: action can be ``accept`` or ``reject``.
        :param endpoints: List of VPC Endpoints Id.

        :returns: A generator of connection objects.
        :rtype: :class:`~otcextensions.sdk.vpcep.connection.Connection`
        """

        endpoint_service = self._get_resource(_service.Service, service)
        connection = self._get_resource(
            _connection.Connection,
            None,
            endpoint_service_id=endpoint_service.id,
        )
        if action in ('accept', 'receive'):
            return connection.accept(self, endpoints)
        elif action == 'reject':
            return connection.reject(self, endpoints)
        raise exceptions.SDKException(
            "Value of action can be 'accept' or 'reject'."
        )

    # ======== Endpoint Service Whitelist ========

    def service_whitelist(self, service, **query):
        """Return a generator of service whitelist

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:

        :returns: A generator of whitelist objects.
        :rtype: :class:`~otcextensions.sdk.vpcep.whitelist.Whitelist`
        """
        if query.get('limit'):
            query.update(paginated=False)
        endpoint_service = self._get_resource(_service.Service, service)
        return self._list(
            _whitelist.Whitelist,
            endpoint_service_id=endpoint_service.id,
            **query
        )

    def manage_service_whitelist(self, service, action, domains=[]):
        """Manage endpoint service whitelist

        :param service: The service can be either the ID or a
            :class:`~otcextensions.sdk.vpcep.service.Service` instance.
        :param action: action can be ``add`` or ``remove``.
        :param domains: List of domains Ids to be added to whitelist.

        :returns: A generator of whitelist objects.
        :rtype: :class:`~otcextensions.sdk.vpcep.whitelist.Whitelist`
        """
        endpoint_service = self._get_resource(_service.Service, service)
        whitelist = self._get_resource(
            _whitelist.Whitelist,
            None,
            endpoint_service_id=endpoint_service.id,
        )
        if action == 'add':
            return whitelist.add(self, domains)
        elif action == 'remove':
            return whitelist.remove(self, domains)
        raise exceptions.SDKException(
            "Value of action can be 'add' or 'remove'."
        )

    # ======== VPCEP Public Service  ========

    def public_services(self, **query):
        """Return a generator of public endpoint services

        :param dict query: Optional query parameters to be sent to limit
            the resources being returned. Valid parameters are:

        :returns: A generator of public endpoint service objects
        :rtype: :class:`~otcextensions.sdk.vpcep.public_service.PublicService`
        """
        if query.get('limit'):
            query.update(paginated=False)
        return self._list(_public_service.PublicService, **query)

    # ======== VPCEP Target Service  ========

    def get_target_service(self, name_or_id, ignore_missing=False):
        """Get a single target endpoint service

        :param name_or_id: The value can be the ID of a endpoint service,
            name of a endpoint service or a
            :class:`~otcextensions.sdk.vpcep.target_service.TargetService`
            instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the resource does not exist.
            When set to ``True``, None will be returned when
            attempting to find a nonexistent resource.

        :returns: One
            :class:`~otcextensions.sdk.vpcep.target_service.TargetService`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
            when no resource can be found.

        """
        try:
            base_path = _target_service.TargetService.base_path + \
                '?id={}'.format(name_or_id)
            return self._get(
                _target_service.TargetService,
                base_path=base_path,
                requires_id=False
            )
        except exceptions.ResourceNotFound:
            pass

        try:
            base_path = _target_service.TargetService.base_path + \
                '?endpoint_service_name={}'.format(name_or_id)
            return self._get(
                _target_service.TargetService,
                base_path=base_path,
                requires_id=False
            )
        except exceptions.ResourceNotFound:
            if ignore_missing:
                return None
            else:
                raise

    # ======== VPCEP Resource Quota  ========

    def resource_quota(self, type=None):
        """Return a generator of vpcep quota

        :param type: Specify the resource type. Value of
            type can be: ``endpoint`` or ``endpoint_service``

        :returns: A generator of quota objects.
        :rtype: :class:`~otcextensions.sdk.vpcep.quota.Quota`
        """
        query = {}
        if type:
            query.update(type=type)
        return self._list(_quota.Quota, **query)



#   Copyright 2013 Nebula Inc.
#
#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
# import datetime
import random
import uuid

import mock
from openstackclient.tests.unit import utils

from otcextensions.sdk.mrs.v1 import cluster
from otcextensions.sdk.mrs.v1 import datasource
from otcextensions.sdk.mrs.v1 import job
from otcextensions.sdk.mrs.v1 import jobbinary
from otcextensions.tests.unit.osclient import test_base


def gen_data(data, columns):
    """Fill expected data tuple based on columns list
    """
    return tuple(getattr(data, attr, "") for attr in columns)


class TestMrs(utils.TestCommand):

    def setUp(self):
        super(TestMrs, self).setUp()

        self.app.client_manager.mrs = mock.Mock()
        self.client = self.app.client_manager.mrs


class FakeHost(test_base.Fake):
    """Fake one or more Host"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": uuid.uuid4().hex,
            "ip": "192.168.0.169",
            "status": "ACTIVE",
            "flavor": "c2.2xlarge.linux.mrs",
            "type": random.choice(["Core", "Master"]),
            "mem": random.randint(1, 10000),
            "cpu": random.randint(1, 10),
            "root_volume_size": random.randint(1, 300),
            "data_volume_type": random.choice(["SATA", "SAS", "SSD"]),
            "data_volume_size": random.randint(1, 1000),
            "data_volume_count": random.randint(1, 5)
        }
        obj = cluster.Host.existing(**object_info)
        return obj


class FakeCluster(test_base.Fake):
    """Fake one or more Cluster"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": uuid.uuid4().hex,
            "master_num": random.randint(1, 5),
            "core_num": random.randint(1, 5),
            "status": random.choice(["starting", "running", "terminated",
                                     "failed", "abnormal", "terminating",
                                     "frozen", "scaling-out", "scaling-in"]
                                    ),
            "created_at": 1487570757,
            "updated_at": 1487668974,
            "billing_type": "Metered",
            "region": "eu-de",
            "vpc": "vpc-autotest",
            "fee": "0",
            "hadoop_version": "",
            "component_list": [{
                "id": None,
                "componentId": "MRS 3.0.2_001",
                "componentName": "Hadoop",
                "componentVersion": "3.1.1",
                "external_datasources": None,
                "componentDesc": "A distributed data processing framework",
                "componentDescEn": None,
            }],
            "master_node_size": "s1.8xlarge.linux.mrs",
            "core_node_size": "s2.2xlarge.linux.mrs",
            "external_ip": "100.120.0.2",
            "external_alternate_ip": "192.120.0.2",
            "internal_ip": "192.120.0.3",
            "deployment_id": "dep_id-" + uuid.uuid4().hex,
            "remark": "",
            "order_id": "null",
            "az_id": "null",
            "az": random.choice(["eu-de-01", "eu-de-02", "eu-de-03"]),
            "master_node_product_id": "m_id-" + uuid.uuid4().hex,
            "master_node_spec_id": "mspec_id-" + uuid.uuid4().hex,
            "core_node_product_id": "c_id-" + uuid.uuid4().hex,
            "core_node_spec_id": "cspec_id-" + uuid.uuid4().hex,
            "instanceId": "inst_id-" + uuid.uuid4().hex,
            "vnc": None,
            "project_id": "proj_id-" + uuid.uuid4().hex,
            "volume_size": random.randint(1, 1000),
            "volume_type": random.choice(["SATA", "SAS", "SSD"]),
            "subnet_id": "sub_id-" + uuid.uuid4().hex,
            "subnet_name": "subnet-ftest",
            "security_group_id": "sec_gr_id-" + uuid.uuid4().hex,
            "non_master_security_group_id": "sec_gr_id_1-" + uuid.uuid4().hex,
            "stage_desc": "Installing MRS Manager",
            "mrs_install_state": False,
            "safe_mode": 1,
            "cluster_version": "MRS 3.0.2",
            "bootstrapScripts": [
                {
                    "name": "test1-success",
                    "uri": "s3a://bootscript/script/simple/basic_success.sh",
                    "parameters": "",
                    "nodes": ["master", "core"],
                    "active_master": True,
                    "fail_action": "errorout",
                    "before_component_start": True,
                    "state": "SUCCESS",
                    "start_time": 1527681083
                }
            ],
            "node_groups": [
                {
                    "groupName": "master_node_default_group",
                    "nodeNum": 1,
                    "nodeSize": "s1.xlarge.linux.mrs",
                    "nodeSpecId": "cdc6035a249a40249312f5ef72a23cd7",
                    "vmProductId": "",
                    "vmSpecCode": None,
                    "nodeProductId": "dc970349d128460e960a0c2b826c427c",
                    "rootVolumeSize": 40,
                    "rootVolumeProductId": "16c1dcf0897249758b1ec276d06e0572",
                    "rootVolumeType": "SATA",
                    "rootVolumeResourceSpecCode": "",
                    "rootVolumeResourceType": "",
                    "dataVolumeType": "SATA",
                    "dataVolumeCount": 1,
                    "dataVolumeSize": 100,
                    "dataVolumeProductId": "16c1dcf0897249758b1ec276d06e0572",
                    "dataVolumeResourceSpecCode": "",
                    "dataVolumeResourceType": ""
                }
            ],
            "task_node_groups": [
                {
                    "groupName": "task_node_default_group",
                    "nodeNum": 1,
                    "nodeSize": "s1.xlarge.linux.mrs",
                    "nodeSpecId": "cdc6035a249a40249312f5ef72a23cd7",
                    "vmProductId": "",
                    "vmSpecCode": None,
                    "nodeProductId": "dc970349d128460e960a0c2b826c427c",
                    "rootVolumeSize": 40,
                    "rootVolumeProductId": "16c1dcf0897249758b1ec276d06e0572",
                    "rootVolumeType": "SATA",
                    "rootVolumeResourceSpecCode": "",
                    "rootVolumeResourceType": "",
                    "dataVolumeType": "SATA",
                    "dataVolumeCount": 1,
                    "dataVolumeSize": 100,
                    "dataVolumeProductId": "16c1dcf0897249758b1ec276d06e0572",
                    "dataVolumeResourceSpecCode": "",
                    "dataVolumeResourceType": "",
                    "AutoScalingPolicy": None
                }
            ],
            "cluster_type": 0,
            "tags": "k1=v1,k2=v2,k3=v3",
            "key": "myp",
            "master_ip": "192.168.1.1",
            "preffered_private_ip": "192.168.1.2",
            "error_info": None,
            "log_collection": 1,
            "master_data_volume_type": random.choice(["SATA", "SAS", "SSD"]),
            "master_data_volume_size": random.randint(1, 1000),
            "master_data_volume_count": random.randint(1, 10),
            "core_data_volume_type": random.choice(["SATA", "SAS", "SSD"]),
            "core_data_volume_size": random.randint(1, 1000),
            "core_data_volume_count": random.randint(1, 10),
        }

        obj = cluster.ClusterInfo.existing(**object_info)
        return obj


class FakeDatasource(test_base.Fake):
    """Fake one or more Datasource"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": uuid.uuid4().hex,
            "type": random.choice(["hdfs", "obs"]),
            "url": "/simple/mapreduce/input",
            "description": "this is the data source template",
            "is_public": random.choice([False, True]),
            "is_protected": random.choice([False, True]),
        }
        obj = datasource.Datasource.existing(**object_info)
        return obj


class FakeJob(test_base.Fake):
    """Fake one or more Job"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": uuid.uuid4().hex,
            "type": random.choice(["MapReduce", "SparkScript",
                                   "Hive", "DistCp"]),
            "description": "This is the Map Reduce job template",
            "created_at": "2017-06-22T09:39:13",
            "updated_at": "2017-06-22T09:39:13",
            "project_id": "pr-" + uuid.uuid4().hex,
            "is_public": False,
            "is_protected": False,
            "interface": [],
            "mains": [],
            "libs": [],
        }
        obj = job.Job.existing(**object_info)
        return obj


class FakeJobbinary(test_base.Fake):
    """Fake one or more Jobbinary"""

    @classmethod
    def generate(cls):
        object_info = {
            "id": "id-" + uuid.uuid4().hex,
            "name": uuid.uuid4().hex,
            "description": "this is the job binary template",
            "is_public": random.choice([False, True]),
            "is_protected": random.choice([False, True]),
            "url": "/simple/mapreduce/program",
            "created_at": "2017-06-22T09:39:13",
            "updated_at": "2017-06-22T09:39:13",
        }
        obj = jobbinary.Jobbinary.existing(**object_info)
        return obj



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.dms.v1 import _base


class Topic(_base.Resource):
    """DMS Topic resource"""
    resources_key = 'topics'
    base_path = '/instances/%(instance_id)s/topics'

    # capabilities
    allow_list = True
    allow_create = True
    allow_delete = True

#    _query_mapping = resource.QueryParameters('instance_id')

    instance_id = resource.URI('instance_id')

    #: Properties
    #: Synchronous flushing. Default=false
    is_sync_flush = resource.Body('sync_message_flush', type=bool)
    #: Synchronous replication. Default=false. With replication=1 can be only
    #: false
    is_sync_replication = resource.Body('sync_replication', type=bool)
    #: Number of partitions. Default=3
    partition = resource.Body('partition', type=int)
    #: Replication factor. Default=3
    replication = resource.Body('replication', type=int)
    #: Retention time in hours. Default=72
    retention_time = resource.Body('retention_time', type=int)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import os
import uuid

import fixtures

from otcextensions.tests.functional import base


class TestDns(base.BaseFunctionalTest):

    def setUp(self):
        super(TestDns, self).setUp()
        test_timeout = 3 * int(os.environ.get('OS_TEST_TIMEOUT'))
        try:
            self.useFixture(
                fixtures.EnvironmentVariable(
                    'OS_TEST_TIMEOUT', str(test_timeout)))
        except ValueError:
            pass
        self.client = self.conn.dns

    def create_network(self):
        cidr = '192.168.0.0/16'
        ipv4 = 4
        uuid_v4 = uuid.uuid4().hex[:8]
        router_name = 'sdk-dns-test-add-router-' + uuid_v4
        net_name = 'sdk-dns-test-add-net-' + uuid_v4
        subnet_name = 'sdk-dns-test-add-subnet-' + uuid_v4

        network = self.conn.network.create_network(name=net_name)
        self.assertEqual(net_name, network.name)
        net_id = network.id
        subnet = self.conn.network.create_subnet(
            name=subnet_name,
            ip_version=ipv4,
            network_id=net_id,
            cidr=cidr
        )
        self.assertEqual(subnet_name, subnet.name)
        subnet_id = subnet.id

        router = self.conn.network.create_router(name=router_name)
        self.assertEqual(router_name, router.name)
        router_id = router.id
        interface = router.add_interface(
            self.conn.network,
            subnet_id=subnet_id
        )
        self.assertEqual(interface['subnet_id'], subnet_id)
        self.assertIn('port_id', interface)
        return {
            'router_id': router_id,
            'subnet_id': subnet_id,
            'network_id': net_id
        }

    def destroy_network(self, params: dict):
        router_id = params.get('router_id')
        subnet_id = params.get('subnet_id')
        network_id = params.get('network_id')
        router = self.conn.network.get_router(router_id)

        interface = router.remove_interface(
            self.conn.network,
            subnet_id=subnet_id
        )
        self.assertEqual(interface['subnet_id'], subnet_id)
        self.assertIn('port_id', interface)
        sot = self.conn.network.delete_router(
            router_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_subnet(
            subnet_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)
        sot = self.conn.network.delete_network(
            network_id,
            ignore_missing=False
        )
        self.assertIsNone(sot)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock
from keystoneauth1 import adapter

from openstack import utils
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv2.v2 import dataset_label
from otcextensions.tests.unit.sdk.modelartsv2.v2 import examples

EXAMPLE = examples.DATASET_LABEL


class TestDatasetLabel(base.TestCase):
    def setUp(self):
        super(TestDatasetLabel, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)

    def test_basic(self):
        sot = dataset_label.DatasetLabel()

        self.assertEqual(
            "/datasets/%(dataset_id)s/data-annotations/labels",
            sot.base_path,
        )
        self.assertEqual(None, sot.resource_key)
        self.assertEqual("labels", sot.resources_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)

        self.assertDictEqual(
            {
                "limit": "limit",
                "marker": "marker",
                "version_id": "version_id",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = dataset_label.DatasetLabel(**EXAMPLE)
        self.assertEqual(sot.name, EXAMPLE["name"])
        self.assertEqual(
            sot.property.color, EXAMPLE["property"]["@modelarts:color"]
        )
        self.assertEqual(
            sot.property.default_shape,
            EXAMPLE["property"]["@modelarts:default_shape"],
        )

    def test_update_labels(self):
        dataset_id = "dataset-id"
        labels = [
            {
                "name": "Cat",
                "property": {
                    "@modelarts:color": "#8a1524",
                    "@modelarts:rename_to": "pussycat",
                },
            }
        ]
        sot = dataset_label.DatasetLabel(dataset_id=dataset_id)
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {"success": True}
        response.headers = {}
        self.sess.put.return_value = response
        rt = sot.update_labels(self.sess, labels)
        self.sess.put.assert_called_with(
            sot.base_path % {"dataset_id": dataset_id}, json={"labels": labels}
        )
        self.assertEqual(rt, sot)

    def test_delete_labels(self):
        dataset_id = "dataset-id"
        labels = [{"name": "Cat"}]
        sot = dataset_label.DatasetLabel(dataset_id=dataset_id)
        response = mock.Mock()
        response.status_code = 200
        response.json.return_value = {"success": True}
        response.headers = {}
        self.sess.post.return_value = response
        rt = sot.delete_labels(self.sess, labels, 2)
        self.sess.post.assert_called_with(
            utils.urljoin(
                sot.base_path % {"dataset_id": dataset_id}, "delete"
            ),
            json={"labels": labels},
            params={"delete_policy": 2},
        )
        self.assertEqual(rt, sot)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''DNS PTR v2 action implementations'''
import logging

from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _
from otcextensions.common import sdk_utils

LOG = logging.getLogger(__name__)


_formatters = {
}


def _get_columns(item):
    column_map = {
    }
    hidden = ['location', 'links']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(item, column_map,
                                                           hidden)


class ListPTR(command.Lister):
    _description = _('List PTR records')
    columns = (
        'id', 'ptrdname', 'address', 'status', 'description', 'ttl'
    )

    def get_parser(self, prog_name):
        parser = super(ListPTR, self).get_parser(prog_name)

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns

        query = {}

        data = client.floating_ips(**query)

        table = (self.columns,
                 (utils.get_item_properties(
                     s, self.columns, formatters=_formatters
                 ) for s in data))
        return table


class ShowPTR(command.ShowOne):
    _description = _('Show the PTR record details')

    def get_parser(self, prog_name):
        parser = super(ShowPTR, self).get_parser(prog_name)

        parser.add_argument(
            'floatingip_id',
            help=_('Floating IP ID in format region:floatingip_id.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        obj = client.get_floating_ip(parsed_args.floatingip_id)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class DeletePTR(command.Command):
    _description = _('Delete (restore) PTR record')

    def get_parser(self, prog_name):
        parser = super(DeletePTR, self).get_parser(prog_name)

        parser.add_argument(
            'floatingip_id',
            help=_('FloatingIP ID.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.dns

        client.unset_floating_ip(parsed_args.floatingip_id)


class SetPTR(command.ShowOne):
    _description = _('Set PTR record')

    def get_parser(self, prog_name):
        parser = super(SetPTR, self).get_parser(prog_name)

        parser.add_argument(
            'floatingip_id',
            help=_('Floating IP ID in format region:floatingip_id.')
        )
        parser.add_argument(
            'ptrdname',
            help=_('PTRD Name')
        )

        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for this record.')
        )
        parser.add_argument(
            '--ttl',
            metavar='<300-2147483647>',
            type=int,
            # NOTE: py2 does not support such big int, skip unless py3-only
            # choices=range(300, 2147483647),
            help=_('TTL (Time to Live) for the zone.')
        )

        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.dns

        attrs = {}

        attrs['ptrdname'] = parsed_args.ptrdname
        if parsed_args.description:
            attrs['description'] = parsed_args.description
        if parsed_args.ttl:
            attrs['ttl'] = parsed_args.ttl

        obj = client.set_floating_ip(
            floating_ip=parsed_args.floatingip_id,
            **attrs
        )

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import exceptions

from otcextensions.sdk import sdk_proxy
from otcextensions.sdk.rds.v1 import backup as _backup
from otcextensions.sdk.rds.v1 import configuration as _configuration
from otcextensions.sdk.rds.v1 import datastore as _datastore
from otcextensions.sdk.rds.v1 import flavor as _flavor
from otcextensions.sdk.rds.v1 import instance as _instance


class Proxy(sdk_proxy.Proxy):

    skip_discovery = True

    def get_os_endpoint(self, **kwargs):
        """Return OpenStack compliant endpoint

        """
        return self.get_endpoint(service_type='database')
        # if 'os_endpoint' not in self:
        #     endpoint = super(Proxy, self).get_endpoint(**kwargs)
        #     # endpoint_override = self.endpoint_override
        #     if endpoint.endswith('/rds/v1'):  # and not endpoint_override:
        #         endpoint = endpoint.rstrip('/rds/v1')
        #         endpoint = utils.urljoin(endpoint, 'v1.0')
        #         self.os_endpoint = endpoint
        #     # else:
        #     #     _logger.debug('RDS endpoint_override is set. Return it')
        #     #     return endpoint_override
        # else:
        #     return self.os_endpoint

    def get_rds_endpoint(self, **kwargs):
        """Return RDS propriatary endpoint

        """
        return self.get_endpoint(service_type='rds')
        # endpoint = super(Proxy, self).get_endpoint(**kwargs)
        # endpoint_override = self.endpoint_override
        # if endpoint.endswith('/rds/v1') and not endpoint_override:
        #     return endpoint
        # elif endpoint_override:
        #     _logger.debug('RDS endpoint_override is set. Return it')
        #     return endpoint_override
        # else:
        #     return endpoint

    def get_os_headers(self, language=None):
        """Get headers for request

        Unfortunatels RDS requires 'Content-Type: application/json'
        header even for GET and LIST operations with empty body
        We need to deel with it

        :param language: whether language should be added to headers
            can be either bool (then self.get_language is used)
            or a language code directly (i.e. "en-us")
        :returns dict: dictionary with headers
        """
        headers = {
            'Content-Type': 'application/json',
        }
        if language:
            if isinstance(language, bool):
                headers['X-Language'] = self.get_language()
            elif isinstance(language, str):
                headers['X-Language'] = language
        return headers

    def get_language(self):
        """Returns language code

        """
        return 'en-us'

    # ======= Datastores =======
    def datastore_types(self):
        """List supported datastore types

        :returns: A generator of supported datastore types
        :rtype object: object with name attribte
        """
        for ds in ['MySQL', 'PostgreSQL', 'SQLServer']:
            obj = type('obj', (object,), {'name': ds})
            yield obj

        return

    def datastore_versions(self, datastore):
        """List datastores

        :param dbId: database store name
            (MySQL, PostgreSQL, or SQLServer and is case-sensitive.)

        :returns: A generator of datastore versions
        :rtype: :class:`~otcextensions.sdk.rds.v1.flavor.Flavor`
        """
        return self._list(
            _datastore.Datastore,
            paginated=False,
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
            # project_id=self.session.get_project_id(),
            datastore_name=datastore
        )

    def get_datastore_version(self, datastore, datastore_version):
        """Get the detail of a datastore version

        :param datastore: datastore name
        :param datastore_Version: id of the datastore version
        :returns: Detail of datastore version
        :rtype: :class:`~otcextensions.sdk.rds.v1.datastore.Datastore`
        """
        versions = self._list(
            _datastore.Datastore,
            paginated=False,
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
            # project_id=self.session.get_project_id(),
            datastore_name=datastore
        )
        for ver in versions:
            if ver.id == datastore_version:
                return ver
        return exceptions.NotFoundException('Resource not found')

    # ======= Flavors =======
    def flavors(self, dbId, region):
        """List flavors of given datastore id and region

        :param dbId: database store id
        :param region: region

        :returns: A generator of flavor
        :rtype: :class:`~otcextensions.sdk.rds_os.v1.flavor.Flavor`
        """
        return self._list(_flavor.Flavor, paginated=False,
                          # endpoint_override=self.get_os_endpoint(),
                          headers=self.get_os_headers(True),
                          dbId=dbId,
                          region=region
                          # project_id=self.session.get_project_id()
                          )

    def get_flavor(self, flavor):
        """Get the detail of a flavor

        :param id: Flavor id or an object of class
                   :class:`~otcextensions.sdk.rds_os.v1.flavor.Flavor`
        :returns: Detail of flavor
        :rtype: :class:`~otcextensions.sdk.rds_os.v1.flavor.Flavor`
        """
        return self._get(
            _flavor.Flavor,
            flavor,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            headers=self.get_os_headers(True),
        )

    # def find_flavor(self, name_or_id, ignore_missing=True):
    #     """Find a single flavor
    #
    #     :param name_or_id: The name or ID of a flavor.
    #     :param bool ignore_missing: When set to ``False``
    #                 :class:`~openstack.exceptions.ResourceNotFound` will be
    #                 raised when the resource does not exist.
    #                 When set to ``True``, None will be returned when
    #                 attempting to find a nonexistent resource.
    #     :returns: One :class:`~otcextensions.sdk.rds.v1.flavor.Flavor`
    #               or None
    #     """
    #     return self._find(_flavor.Flavor, name_or_id,
    #                       # project_id=self.session.get_project_id(),
    #                       # endpoint_override=self.get_os_endpoint(),
    #                       # headers=self.get_os_headers(),
    #                       ignore_missing=ignore_missing)

    # ======= Instance =======
    def create_instance(self, **attrs):
        """Create a new instance from attributes

        :param dict attrs: Keyword arguments which will be used to create
                   a :class:`~otcextensions.sdk.rds.v1.instance.Instance`,
                   comprised of the properties on the Instance class.

        :returns: The results of server creation
        :rtype: :class:`~otcextensions.sdk.rds.v1.instance.Instance`
        """
        return self._create(_instance.Instance,
                            # project_id=self.session.get_project_id(),
                            # endpoint_override=self.get_os_endpoint(),
                            headers=self.get_os_headers(True),
                            **attrs)

    def delete_instance(self, instance, ignore_missing=True):
        """Delete an instance

        :param instance: The value can be either the ID of an instance or a
            :class:`~otcextensions.sdk.rds.v1.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent instance.

        :returns: ``None``
        """
        self._delete(
            _instance.Instance, instance,
            ignore_missing=ignore_missing,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            headers=self.get_os_headers(True)
        )

    def find_instance(self, name_or_id, ignore_missing=True):
        """Find a single instance

        :param name_or_id: The name or ID of a instance.
        :param bool ignore_missing: When set to ``False``
                    :class:`~openstack.exceptions.ResourceNotFound` will be
                    raised when the resource does not exist.
                    When set to ``True``, None will be returned when
                    attempting to find a nonexistent resource.
        :returns: One :class:`~otcextensions.sdk.rds.v1.instance.Instance`
                  or None
        """
        return self._find(_instance.Instance, name_or_id,
                          # project_id=self.session.get_project_id(),
                          # endpoint_override=self.get_os_endpoint(),
                          headers=self.get_os_headers(True),
                          ignore_missing=ignore_missing)

    def get_instance(self, instance):
        """Get a single instance

        :param instance: The value can be the ID of an instance or a
                         :class:`~otcextensions.sdk.rds.v1.instance.Instance`
                         instance.

        :returns: One :class:`~otcextensions.sdk.rds.v1.instance.Instance`
        :raises: :class:`~openstack.exceptions.ResourceNotFound`
                 when no resource can be found.
        """
        return self._get(
            _instance.Instance,
            instance,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            headers=self.get_os_headers(True),
        )

    def instances(self):
        """Return a generator of instances

        :returns: A generator of instance objects
        :rtype: :class:`~otcextensions.sdk.rds.v1.instance.Instance`
        """
        return self._list(
            _instance.Instance, paginated=False,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            headers=self.get_os_headers(True),
        )

    def update_instance(self, instance, **attrs):
        """Update a instance

        :param instance: Either the id of a instance or a
                         :class:`~otcextensions.sdk.rds.v1.instance.Instance`
                         instance.
        :attrs attrs: The attributes to update on the instance represented
                       by ``value``.

        :returns: The updated instance
        :rtype: :class:`~otcextensions.sdk.rds.v1.instance.Instance`
        """
        return self._update(
            _instance.Instance,
            instance=instance,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
            **attrs
        )

    def restart_instance(self, instance):
        """Restart instance

        :param instance: Either the id of a instance or a
                         :class:`~otcextensions.sdk.rds.v1.instance.Instance`
                         instance.
        :returns: The updated instance
        :rtype: :class:`~otcextensions.sdk.rds.v1.instance.Instance`
        """
        instance = self._get_resource(_instance.Instance, instance)
        return instance.restart(self)

    def restore_instance(self, instance, backup):
        """Restore instance from backup

        :param instance: Either the id of a instance or a
                         :class:`~otcextensions.sdk.rds.v1.instance.Instance`
                         instance.
        :param backup: Either the id of a backup or a
                         :class:`~otcextensions.sdk.rds.v1.backup.Backup`
                         instance.

        :returns: None
        :rtype:
        """
        instance = self._get_resource(_instance.Instance, instance)
        backup = self._get_resource(_backup.Backup, backup)
        return instance.restore(self, backupRef=backup.id)
    #
    # def create_instance_from_backup(self, backup, **attrs):
    #     """Restore instance from backup
    #
    #     :param instance: Either the id of a instance or a
    #                      :class:`~otcextensions.sdk.rds.v1.instance.Instance`
    #                      instance.
    #     :param backup: Either the id of a backup or a
    #                      :class:`~otcextensions.sdk.rds.v1.backup.Backup`
    #                      instance.
    #     :attrs attrs: The attributes to update on the instance represented
    #                    by ``value``.
    #
    #     :returns: The updated instance
    #     :rtype: :class:`~otcextensions.sdk.rds.v1.instance.Instance`
    #     """
    #     # instance = self._get_resource(_instance.Instance, instance)
    #     # backup = self._get_resource(_backup.Backup, backup)
    #
    #     res = _instance.Instance.new(**attrs)
    #
    #     return res.create(self)

    # ======= Configurations =======
    def configurations(self, **attrs):
        """Obtaining a ConfigurationGroup List

        :returns: A generator of ConfigurationGroup object
        :rtype:
            :class:`~otcextensions.sdk.rds.v1.configuration.ConfigurationGroup`
        """
        return self._list(
            _configuration.ConfigurationGroup,
            paginated=False,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            # headers=self.get_os_headers(),
        )

    def get_configuration(self, cg):
        """Obtaining a ConfigurationGroup

        :param parameter_group: The value can be the ID of a ConfigurationGroup
            or a object of
            :class:`~otcextensions.sdk.rds.v1.configuration.ConfigurationGroup`.

        :returns: A Parameter Group Object
        :rtype: :class:`~otcextensions.rds.v1.configuration.ConfigurationGroup`
        """
        return self._get(
            _configuration.ConfigurationGroup,
            cg,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            headers=self.get_os_headers(True)
        )

    def create_configuration(self, **attrs):
        """Creating a ConfigurationGroup

        :param dict **attrs: Dict to overwrite ConfigurationGroup object
        :returns: A Parameter Group Object
        :rtype:
            :class:`~otcextensions.sdk.rds.v1.configuration.ConfigurationGroup`
        """
        return self._create(_configuration.ConfigurationGroup,
                            # project_id=self.session.get_project_id(),
                            # endpoint_override=self.get_os_endpoint(),
                            # headers=self.get_os_headers(),
                            **attrs)

    def delete_configuration(self, cg, ignore_missing=True):
        """Deleting a ConfigurationGroup

        :param cg: The value can be the ID of a ConfigurationGroup or a
            object of
            :class:`~otcextensions.sdk.rds.v1.configuration.ConfigurationGroup`.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the Parameter Group does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent Parameter Group.

        :returns: None
        :rtype: None
        """
        self._delete(
            _configuration.ConfigurationGroup, cg,
            ignore_missing=ignore_missing,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_os_endpoint(),
            # headers=self.get_os_headers()
        )

    def find_configuration(self, name_or_id, ignore_missing=True):
        """Find a ConfigurationGroup

        :param parameter_group: The value can be the ID of a ConfigurationGroup
            or a object of
            :class:`~otcextensions.sdk.rds.v1.configuration.Configurations`.
        :returns: A Parameter Group Object
        :rtype:
            :class:`~otcextensions.rds.v1.configuration.ConfigurationGroup`.
        """
        return self._find(_configuration.ConfigurationGroup, name_or_id,
                          # project_id=self.session.get_project_id(),
                          # endpoint_override=self.get_os_endpoint(),
                          # headers=self.get_os_headers(),
                          ignore_missing=ignore_missing)

    # ======= Backups =======
    def backups(self):
        """List Backups

        :returns: A generator of backup
        :rtype: :class:`~otcextensions.sdk.rds.v1.backup.Backup`
        """
        return self._list(
            _backup.Backup, paginated=False,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True)
        )

    def create_backup(self, instance, **attrs):
        """Create a backups of instance

        :returns: A new backup object
        :rtype: :class:`~otcextensions.sdk.rds.v1.backup.Backup`
        """
        instance = self._get_resource(_instance.Instance, instance)
        return self._create(
            _backup.Backup,
            instance=instance.id,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
            **attrs
        )

    def delete_backup(self, backup, ignore_missing=True):
        """Deletes given backup

        :param instance: The value can be either the ID of an instance or a
            :class:`~openstack.database.v1.instance.Instance` instance.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be
            raised when the instance does not exist.
            When set to ``True``, no exception will be set when
            attempting to delete a nonexistent instance.

        :returns: ``None``
        """
        return self._delete(
            _backup.Backup,
            backup,
            ignore_missing=ignore_missing,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
        )

    def get_backup_policy(self, instance):
        """Obtaining a backup policy of the instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v1.instance.Instance`
        :returns: A Backup policy
        :rtype: :class:`~otcextensions.sdk.rds.v1.configuration.BackupPolicy`

        """
        instance = self._get_resource(_instance.Instance, instance)
        return self._get(
            _backup.BackupPolicy,
            requires_id=False,
            instance_id=instance.id,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
        )

    def set_backup_policy(self, backup_policy, instance, **attrs):
        """Sets the backup policy of the instance

        :param instance: This parameter can be either the ID of an instance
            or a :class:`~openstack.sdk.rds.v1.instance.Instance`
        :param dict attrs: The attributes to update on the backup_policy
            represented by ``backup_policy``.

        :returns: ``None``
        """
        instance = self._get_resource(_instance.Instance, instance)
        return self._update(
            _backup.BackupPolicy,
            backup_policy,
            instance_id=instance.id,
            # project_id=self.session.get_project_id(),
            # endpoint_override=self.get_rds_endpoint(),
            headers=self.get_os_headers(True),
            **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""VPC Endpoint Service v1 action implementations"""
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.cli import format_columns
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.common import cli_utils
from otcextensions.common import sdk_utils
from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


_formatters = {
    'active_status': format_columns.ListColumn,
    'tags': cli_utils.YamlFormat,
    'whitelist': format_columns.ListColumn,
    'route_tables': format_columns.ListColumn,
}


def _get_columns(item):
    column_map = {}
    hidden = [
        'location',
    ]
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden
    )


class ListEndpoints(command.Lister):

    _description = _('List VPC Endpoints.')
    columns = ('Id', 'Endpoint Service Name', 'Status', 'Is Enabled')

    def get_parser(self, prog_name):
        parser = super(ListEndpoints, self).get_parser(prog_name)

        parser.add_argument(
            '--id',
            metavar='<id>',
            help=_('ID of the VPC endpoint.'),
        )
        parser.add_argument(
            '--service-name',
            metavar='<service_name>',
            dest='endpoint_service_name',
            help=_('Name of the VPC endpoint service.'),
        )
        parser.add_argument(
            '--sort-key',
            metavar='{created_at, updated_at}',
            type=lambda s: s.lower(),
            choices=['created_at', 'updated_at'],
            help=_('Sorting field of the VPC endpoint list.'),
        )
        parser.add_argument(
            '--sort-dir',
            metavar='{asc, desc}',
            type=lambda s: s.lower(),
            choices=['asc', 'desc'],
            help=_('Sorting order of the VPC endpoint list.'),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Limit number of VPC endpoints.'),
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_('Endpoints after this offset will be queried.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        args_list = [
            'id',
            'endpoint_service_name',
            'limit',
            'offset',
            'sort_key',
            'sort_dir',
        ]
        params = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                params[arg] = val

        data = client.endpoints(**params)

        return (
            self.columns,
            (utils.get_item_properties(s, self.columns) for s in data),
        )


class CreateEndpoint(command.ShowOne):
    _description = _(
        'Create a VPC endpoint for accessing a VPC endpoint service.'
    )

    def get_parser(self, prog_name):
        parser = super(CreateEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            '--service-id',
            metavar='<service_id>',
            dest='endpoint_service_id',
            help=_('ID of the Vpc endpoint service.'),
        )
        parser.add_argument(
            '--router-id',
            metavar='<router_id>',
            dest='vpc_id',
            required=True,
            help=_(
                'ID of the vpc/router where the VPC endpoint is to be created.'
            ),
        )
        parser.add_argument(
            '--network-id',
            metavar='<network_id>',
            dest='subnet_id',
            required=True,
            help=_('ID of the network created in the vpc/router.'),
        )
        parser.add_argument(
            '--port-ip',
            metavar='<port_ip>',
            help=_(
                'IP address for accessing the associated VPC endpoint service.'
            ),
        )
        parser.add_argument(
            '--route-tables',
            metavar='<route_tables>',
            dest='routetables',
            nargs='+',
            help=_('Lists the IDs of route tables.'),
        )
        parser.add_argument(
            '--whitelist',
            metavar='<whitelist>',
            nargs='+',
            help=_('Whitelist for controlling access to the VPC endpoint.'),
        )
        parser.add_argument(
            '--specification-name',
            metavar='<specification_name>',
            help=_('Name of the VPC endpoint specifications.'),
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description of the VPC endpoint.'),
        )
        parser.add_argument(
            '--tags',
            metavar='key=<tag-key>,value=<tag-value>',
            action=parseractions.MultiKeyValueAction,
            dest='tags',
            required_keys=['key', 'value'],
            help=_(
                'Example: \n'
                '--tags key=test-key,value=test-value\n'
                'Repeat option to provide multiple tags.'
            ),
        )
        parser.add_argument(
            '--enable-dns',
            action='store_true',
            help=('Whether to create a private domain name. default (false)'),
        )
        parser.add_argument(
            '--enable-whitelist',
            action='store_true',
            help=('Whether access control is enabled.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep

        args_list = [
            'endpoint_service_id',
            'subnet_id',
            'vpc_id',
            'port_ip',
            'tags',
            'whitelist',
            'routetables',
            'enable_dns',
            'enable_whitelist',
            'specification_name',
            'description',
        ]
        attrs = {}
        for arg in args_list:
            val = getattr(parsed_args, arg)
            if val:
                attrs[arg] = val

        obj = client.create_endpoint(**attrs)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns)

        return (display_columns, data)


class ShowEndpoint(command.ShowOne):
    _description = _('Show VPC endpoint details.')

    def get_parser(self, prog_name):
        parser = super(ShowEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            help=_('ID of the VPC endpoint.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        obj = client.get_endpoint(parsed_args.endpoint)
        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns, formatters=_formatters)

        return (display_columns, data)


class DeleteEndpoint(command.Command):

    _description = _('Delete VPC endpoint(s).')

    def get_parser(self, prog_name):
        parser = super(DeleteEndpoint, self).get_parser(prog_name)
        parser.add_argument(
            'endpoint',
            metavar='<endpoint>',
            nargs='+',
            help=_('ID of vpc endpoint(s) to delete.'),
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.vpcep
        result = 0
        for endpoint in parsed_args.endpoint:
            try:
                client.delete_endpoint(endpoint)
            except Exception as e:
                result += 1
                LOG.error(
                    _(
                        'Failed to delete VPC endpoint with '
                        'ID "%(endpoint)s": %(e)s'
                    ),
                    {'endpoint': endpoint, 'e': e},
                )
        if result > 0:
            total = len(parsed_args.endpoint)
            msg = _(
                '%(result)s of %(total)s VPC endpoint(s) failed to delete.'
            ) % {'result': result, 'total': total}
            raise exceptions.CommandError(msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.vlb.v3 import member

EXAMPLE = {
    'protocol_port': 80,
    'name': 'name',
    'address': '192.168.1.3',
}


class TestLoadBalancer(base.TestCase):

    def test_basic(self):
        sot = member.Member()
        path = '/elb/pools/%(pool_id)s/members'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)

    def test_make_it(self):
        sot = member.Member(**EXAMPLE)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['address'], sot.address)
        self.assertEqual(EXAMPLE['protocol_port'], sot.protocol_port)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import mock

from otcextensions.osclient.css.v1 import flavor
from otcextensions.tests.unit.osclient.css.v1 import fakes


class TestListFlavors(fakes.TestCss):

    objects = fakes.FakeFlavor.create_multiple(3)

    column_list_headers = (
        'Id',
        'Name',
        'Version',
        'Type',
        'Availability Zones',
        'Disk Range',
        'vCPUs',
        'RAM',
    )

    columns = (
        'id',
        'name',
        'version',
        'type',
        'availability_zones',
        'disk_range',
        'vcpus',
        'ram',
    )

    data = []

    for s in objects:
        data.append(
            (
                s.id,
                s.name,
                s.version,
                s.type,
                s.availability_zones,
                s.disk_range,
                s.vcpus,
                s.ram,
            )
        )

    def setUp(self):
        super(TestListFlavors, self).setUp()

        self.cmd = flavor.ListFlavors(self.app, None)

        self.client.flavors = mock.Mock()
        self.client.api_mock = self.client.flavors

    def test_list(self):
        arglist = []

        verifylist = []

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))

    def test_list_args(self):
        arglist = [
            '--datastore-version',
            '7.6.2',
            '--node-type',
            'ess',
        ]

        verifylist = [
            ('datastore_version', '7.6.2'),
            ('node_type', 'ess'),
        ]

        # Verify cm is triggered with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [self.objects]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with()

        self.assertEqual(self.column_list_headers, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class AclPolicy(resource.Resource):
    base_path = f'apigw/instances/%(gateway_id)s/acls'
    resources_key = 'acls'

    allow_create = True
    allow_commit = True
    allow_delete = True
    allow_list = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        'limit', 'offset', 'name', 'id', 'precise_search',
        'acl_type', 'entity_type'
    )

    gateway_id = resource.URI('gateway_id', type=str)
    id = resource.Body('id')
    acl_name = resource.Body('acl_name')
    acl_type = resource.Body('acl_type')
    acl_value = resource.Body('acl_value')
    entity_type = resource.Body('entity_type')
    update_time = resource.Body('update_time')
    bind_num = resource.Body('bind_num')
    error_code = resource.Body('error_code')
    error_msg = resource.Body('error_msg')

    def _delete_multiple_acls(self, session, gateway_id, **attrs):
        uri = f'apigw/instances/{gateway_id}/acls?action=DELETE'
        response = session.put(uri, json=attrs)
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import copy

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.cce.v3 import cluster_cert


EXAMPLE = {
    "kind": "Config",
    "apiVersion": "v1",
    "preferences": {},
    "clusters": [
        {
            "name": "internalCluster",
            "cluster": {
                "server": "https://192.168.1.7:5443",
                "certificate-authority-data": "ca-data"
            }
        }
    ],
    "users": [
        {
            "name": "user",
            "user": {
                "client-certificate-data": "cc-data",
                "client-key-data": "ck-data"
            }
        }
    ],
    "contexts": [
        {
            "name": "internal",
            "context": {
                "cluster": "internalCluster",
                "user": "user"
            }
        }
    ],
    "current-context": "internal"
}


class TestClusterCert(base.TestCase):

    def setUp(self):
        super(TestClusterCert, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = '1'
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = cluster_cert.ClusterCertificate()
        self.assertEqual(None, sot.resource_key)
        self.assertEqual(None, sot.resources_key)
        self.assertEqual('/clusters/%(cluster_id)s/clustercert',
                         sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertFalse(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertFalse(sot.allow_commit)
        self.assertFalse(sot.allow_delete)

    def test_make_it(self):
        obj = copy.deepcopy(EXAMPLE)
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = obj
        mock_response.headers = {}

        sot = cluster_cert.ClusterCertificate()
        sot._translate_response(mock_response)
        self.assertEqual(
            obj['clusters'][0]['cluster']['certificate-authority-data'],
            sot.ca)
        self.assertEqual(
            obj['users'][0]['user']['client-certificate-data'],
            sot.client_certificate)
        self.assertEqual(
            obj['users'][0]['user']['client-key-data'],
            sot.client_key)
        self.assertEqual(
            {
                'name': obj['contexts'][0]['name'],
                'cluster': obj['clusters'][0]['cluster']['server'],
                'user': obj['contexts'][0]['context']['user']
            },
            sot.context)
